@@ -36,7 +36,7 @@ import com.fasterxml.jackson.databind.util.NameTransformer;
 @SuppressWarnings("serial")
 public abstract class AnnotationIntrospector
     implements Versioned, java.io.Serializable
-{    
+{
     /*
     /**********************************************************
     /* Helper types
@@ -156,17 +156,6 @@ public abstract class AnnotationIntrospector
     /* Meta-annotations (annotations for annotation types)
     /**********************************************************
      */
-    
-    /**
-     * Method called by framework to determine whether given annotation
-     * is handled by this introspector.
-     *
-     * @deprecated Not used since 2.0; deprecated since 2.1
-     */
-    @Deprecated
-    public boolean isHandled(Annotation ann) {
-        return false;
-    }
 
     /**
      * Method for checking whether given annotation is considered an
@@ -242,9 +231,7 @@ public abstract class AnnotationIntrospector
     /**
      * Method for checking whether an annotation indicates that all unknown properties
      */
-    public Boolean findIgnoreUnknownProperties(AnnotatedClass ac) {
-        return null;
-    }
+    public Boolean findIgnoreUnknownProperties(AnnotatedClass ac) { return null; }
 
     /**
      * Method for checking whether properties that have specified type
@@ -257,9 +244,13 @@ public abstract class AnnotationIntrospector
      *   Boolean.FALSE if they are not to be ignored, null for default
      *   handling (which is 'do not ignore')
      */
-    public Boolean isIgnorableType(AnnotatedClass ac) {
-        return null;
-    }
+    public Boolean isIgnorableType(AnnotatedClass ac) { return null; }
+
+    /**
+     * @deprecated Since 2.3: use the one that takes generic {@link Annotated}
+     */
+    @Deprecated
+    public Object findFilterId(AnnotatedClass ac) { return findFilterId((Annotated) ac); }
 
     /**
      * Method for finding if annotated class has associated filter; and if so,
@@ -268,9 +259,7 @@ public abstract class AnnotationIntrospector
      * @return Id of the filter to use for filtering properties of annotated
      *    class, if any; or null if none found.
      */
-    public Object findFilterId(AnnotatedClass ac) {
-        return null;
-    }
+    public Object findFilterId(Annotated ann) { return null; }
 
     /**
      * Method for finding {@link PropertyNamingStrategy} for given
@@ -283,9 +272,7 @@ public abstract class AnnotationIntrospector
      * 
      * @since 2.1
      */
-    public Object findNamingStrategy(AnnotatedClass ac) {
-        return null;
-    }    
+    public Object findNamingStrategy(AnnotatedClass ac) { return null; }
 
     /*
     /**********************************************************
@@ -300,8 +287,7 @@ public abstract class AnnotationIntrospector
      * (if no annotations are found), or build and return a derived instance (using
      * checker's build methods).
      */
-    public VisibilityChecker<?> findAutoDetectVisibility(AnnotatedClass ac,
-            VisibilityChecker<?> checker) {
+    public VisibilityChecker<?> findAutoDetectVisibility(AnnotatedClass ac, VisibilityChecker<?> checker) {
         return checker;
     }
     
@@ -381,18 +367,14 @@ public abstract class AnnotationIntrospector
      * 
      * @param a Annotated entity (class, field/method) to check for annotations
      */
-    public List<NamedType> findSubtypes(Annotated a) {
-        return null;
-    }
+    public List<NamedType> findSubtypes(Annotated a) { return null; }
 
     /**
      * Method for checking if specified type has explicit name.
      * 
      * @param ac Class to check for type name annotations
      */
-    public String findTypeName(AnnotatedClass ac) {
-        return null;
-    }
+    public String findTypeName(AnnotatedClass ac) { return null; }
     
     /*
     /**********************************************************
@@ -404,9 +386,7 @@ public abstract class AnnotationIntrospector
      * Method for checking if given member indicates that it is part
      * of a reference (parent/child).
      */
-    public ReferenceProperty findReferenceType(AnnotatedMember member) {
-        return null;
-    }
+    public ReferenceProperty findReferenceType(AnnotatedMember member) { return null; }
 
     /**
      * Method called to check whether given property is marked to be "unwrapped"
@@ -416,9 +396,7 @@ public abstract class AnnotationIntrospector
      * should  be done, or null if not -- note that transformation may simply
      * be identity transformation (no changes).
      */
-    public NameTransformer findUnwrappingNameTransformer(AnnotatedMember member) {
-        return null;
-    }
+    public NameTransformer findUnwrappingNameTransformer(AnnotatedMember member) { return null; }
 
     /**
      * Method called to check whether given property is marked to
@@ -427,9 +405,7 @@ public abstract class AnnotationIntrospector
      * annotations from multiple accessors (getters, setters, fields,
      * constructor parameters).
      */
-    public boolean hasIgnoreMarker(AnnotatedMember m) {
-        return false;
-    }
+    public boolean hasIgnoreMarker(AnnotatedMember m) { return false; }
 
     /**
      * Method called to find out whether given member expectes a value
@@ -444,9 +420,7 @@ public abstract class AnnotationIntrospector
      * @return Identifier of value to inject, if any; null if no injection
      *   indicator is found
      */
-    public Object findInjectableValueId(AnnotatedMember m) {
-        return null;
-    }
+    public Object findInjectableValueId(AnnotatedMember m) { return null; }
 
     /**
      * Method that can be called to check whether this member has
@@ -455,9 +429,7 @@ public abstract class AnnotationIntrospector
      * 
      * @since 2.0
      */
-    public Boolean hasRequiredMarker(AnnotatedMember m) {
-        return null;
-    }
+    public Boolean hasRequiredMarker(AnnotatedMember m) { return null; }
     
     /**
      * Method for checking if annotated property (represented by a field or
@@ -472,24 +444,7 @@ public abstract class AnnotationIntrospector
      * @return Array of views (represented by classes) that the property is included in;
      *    if null, always included (same as returning array containing <code>Object.class</code>)
      */
-    public Class<?>[] findViews(Annotated a) {
-        return null;
-    }
-
-    /**
-     * Method for finding format annotations for given member.
-     * Return value is typically used by serializers and/or
-     * deserializers to customize presentation aspects of the
-     * serialized value.
-     * 
-     * @since 2.0
-     * 
-     * @deprecated Since 2.1, use {@link #findFormat(Annotated)} instead.
-     */
-    @Deprecated
-    public JsonFormat.Value findFormat(AnnotatedMember member) {
-        return null;
-    }
+    public Class<?>[] findViews(Annotated a) { return null; }
 
     /**
      * Method for finding format annotations for property or class.
@@ -499,24 +454,14 @@ public abstract class AnnotationIntrospector
      * 
      * @since 2.1
      */
-    public JsonFormat.Value findFormat(Annotated memberOrClass) {
-        if (memberOrClass instanceof AnnotatedMember) {
-            return findFormat((AnnotatedMember) memberOrClass);
-        }
-        return null;
-    }
+    public JsonFormat.Value findFormat(Annotated memberOrClass) { return null; }
     
     /**
      * Method for checking whether given accessor claims to represent
      * type id: if so, its value may be used as an override,
      * instead of generated type id.
-     * 
-     * @since 2.0
      */
-    public Boolean isTypeId(AnnotatedMember member) {
-        return null;
-    }
-
+    public Boolean isTypeId(AnnotatedMember member) { return null; }
 
     /**
      * Method used to check if specified property has annotation that indicates
@@ -529,9 +474,47 @@ public abstract class AnnotationIntrospector
      * 
      * @since 2.1
      */
-    public PropertyName findWrapperName(Annotated ann) {
-        return null;
-    }
+    public PropertyName findWrapperName(Annotated ann) { return null; }
+
+    /**
+     * Method used to check whether specified property member (accessor
+     * or mutator) defines human-readable description to use for documentation.
+     * There are no further definitions for contents; for example, whether
+     * these may be marked up using HTML is not defined.
+     * 
+     * @return Human-readable description, if any.
+     * 
+     * @since 2.3
+     */
+    public String findPropertyDescription(Annotated ann) { return null; }
+
+    /**
+     * Method used to check whether specified property member (accessor
+     * or mutator) defines numeric index, and if so, what is the index value.
+     * Possible use cases for index values included use by underlying data format
+     * (some binary formats mandate use of index instead of name) and ordering
+     * of properties (for documentation, or during serialization).
+     * 
+     * @since 2.4
+     * 
+     * @return Explicitly specified index for the property, if any
+     */
+    public Integer findPropertyIndex(Annotated ann) { return null; }
+
+    /**
+     * Method for finding implicit name for a property that given annotated
+     * member (field, method, creator parameter) may represent.
+     * This is different from explicit, annotation-based property name, in that
+     * it is "weak" and does not either proof that a property exists (for example,
+     * if visibility is not high enough), or override explicit names.
+     * In practice this method is used to introspect optional names for creator
+     * parameters (which may or may not be available and can not be detected
+     * by standard databind); or to provide alternate name mangling for
+     * fields, getters and/or setters.
+     * 
+     * @since 2.4
+     */
+    public String findImplicitPropertyName(AnnotatedMember member) { return null; }
     
     /*
     /**********************************************************
@@ -572,6 +555,16 @@ public abstract class AnnotationIntrospector
     public Object findContentSerializer(Annotated am) {
         return null;
     }
+
+    /**
+     * Method for getting a serializer definition for serializer to use
+     * for nulls (null values) of associated property or type.
+     * 
+     * @since 2.3
+     */
+    public Object findNullSerializer(Annotated am) {
+        return null;
+    }
     
     /**
      * Method for checking whether given annotated entity (class, method,
@@ -702,6 +695,11 @@ public abstract class AnnotationIntrospector
      * for which no explicit is defined should be alphabetically (lexicograpically)
      * ordered
      */
+    public Boolean findSerializationSortAlphabetically(Annotated ann) {
+        return null;
+    }
+
+    @Deprecated // since 2.4
     public Boolean findSerializationSortAlphabetically(AnnotatedClass ac) {
         return null;
     }
@@ -726,61 +724,18 @@ public abstract class AnnotationIntrospector
      * 
      * @since 2.1
      */
-    public PropertyName findNameForSerialization(Annotated a)
-    {
-        // [Issue#69], need bit of delegation 
-        // !!! TODO: in 2.2, remove old methods?
-        String name;
-        if (a instanceof AnnotatedField) {
-            name = findSerializationName((AnnotatedField) a);
-        } else if (a instanceof AnnotatedMethod) {
-            name = findSerializationName((AnnotatedMethod) a);
-        } else {
-            name = null;
-        }
+    public PropertyName findNameForSerialization(Annotated a) {
+        /*
         if (name != null) {
             if (name.length() == 0) { // empty String means 'default'
                 return PropertyName.USE_DEFAULT;
             }
             return new PropertyName(name);
         }
-        return null;
-    }
-    
-    /**
-     * Method for checking whether given method has an annotation
-     * that suggests property name associated with method that
-     * may be a "getter". Should return null if no annotation
-     * is found; otherwise a non-null String.
-     * If non-null value is returned, it is used as the property
-     * name, except for empty String ("") which is taken to mean
-     * "use standard bean name detection if applicable;
-     * method name if not".
-     * 
-     * @deprecated Since 2.1 should use {@link #findNameForSerialization} instead
-     */
-    @Deprecated
-    public String findSerializationName(AnnotatedMethod am) {
+        */
         return null;
     }
 
-    /**
-     * Method for checking whether given member field represent
-     * a serializable logical property; and if so, returns the
-     * name of that property.
-     * Should return null if no annotation is found (indicating it
-     * is not a serializable field); otherwise a non-null String.
-     * If non-null value is returned, it is used as the property
-     * name, except for empty String ("") which is taken to mean
-     * "use the field name as is".
-     * 
-     * @deprecated Since 2.1 should use {@link #findNameForSerialization} instead
-     */
-    @Deprecated
-    public String findSerializationName(AnnotatedField af) {
-        return null;
-    }
-    
     /**
      * Method for checking whether given method has an annotation
      * that suggests that the return value of annotated method
@@ -1000,73 +955,15 @@ public abstract class AnnotationIntrospector
      * 
      * @since 2.1
      */
-    public PropertyName findNameForDeserialization(Annotated a)
-    {
-        // [Issue#69], need bit of delegation 
-        // !!! TODO: in 2.2, remove old methods?
-        String name;
-        if (a instanceof AnnotatedField) {
-            name = findDeserializationName((AnnotatedField) a);
-        } else if (a instanceof AnnotatedMethod) {
-            name = findDeserializationName((AnnotatedMethod) a);
-        } else if (a instanceof AnnotatedParameter) {
-            name = findDeserializationName((AnnotatedParameter) a);
-        } else {
-            name = null;
-        }
+    public PropertyName findNameForDeserialization(Annotated a) {
+        /*
         if (name != null) {
             if (name.length() == 0) { // empty String means 'default'
                 return PropertyName.USE_DEFAULT;
             }
             return new PropertyName(name);
         }
-        return null;
-    }
-    
-    /**
-     * Method for checking whether given method has an annotation
-     * that suggests property name associated with method that
-     * may be a "setter". Should return null if no annotation
-     * is found; otherwise a non-null String.
-     * If non-null value is returned, it is used as the property
-     * name, except for empty String ("") which is taken to mean
-     * "use standard bean name detection if applicable;
-     * method name if not".
-     * 
-     * @deprecated Since 2.1 should use {@link #findNameForDeserialization} instead
-     */
-    @Deprecated
-    public String findDeserializationName(AnnotatedMethod am) {
-        return null;
-    }
-
-    /**
-     * Method for checking whether given member field represent
-     * a deserializable logical property; and if so, returns the
-     * name of that property.
-     * Should return null if no annotation is found (indicating it
-     * is not a deserializable field); otherwise a non-null String.
-     * If non-null value is returned, it is used as the property
-     * name, except for empty String ("") which is taken to mean
-     * "use the field name as is".
-     * 
-     * @deprecated Since 2.1 should use {@link #findNameForDeserialization} instead
-     */
-    @Deprecated
-    public String findDeserializationName(AnnotatedField af) {
-        return null;
-    }
-
-    /**
-     * Method for checking whether given set of annotations indicates
-     * property name for associated parameter.
-     * No actual parameter object can be passed since JDK offers no
-     * representation; just annotations.
-     * 
-     * @deprecated Since 2.1 should use {@link #findNameForDeserialization} instead
-     */
-    @Deprecated
-    public String findDeserializationName(AnnotatedParameter param) {
+        */
         return null;
     }
     
@@ -1109,27 +1006,4 @@ public abstract class AnnotationIntrospector
     public boolean hasCreatorAnnotation(Annotated a) {
         return false;
     }
-
-    /*
-    /**********************************************************
-    /* Helper classes
-    /**********************************************************
-     */
-
-    /**
-     * Old version of {@link AnnotationIntrospectorPair}.
-     * 
-     * @deprecated Starting with 2.1, use {@link AnnotationIntrospectorPair} instead.
-     */
-    @Deprecated
-    public static class Pair
-        extends AnnotationIntrospectorPair
-    {
-        private static final long serialVersionUID = 1L;
-
-        @Deprecated
-        public Pair(AnnotationIntrospector p, AnnotationIntrospector s) {
-            super(p, s);
-        }
-    }
 }
