@@ -3,16 +3,19 @@ package com.fasterxml.jackson.databind;
 import java.io.*;
 import java.text.DateFormat;
 import java.util.Locale;
+import java.util.Map;
 import java.util.TimeZone;
+import java.util.concurrent.atomic.AtomicReference;
 
 import com.fasterxml.jackson.core.*;
+import com.fasterxml.jackson.core.io.CharacterEscapes;
 import com.fasterxml.jackson.core.io.SegmentedStringWriter;
 import com.fasterxml.jackson.core.type.TypeReference;
 import com.fasterxml.jackson.core.util.ByteArrayBuilder;
 import com.fasterxml.jackson.core.util.DefaultPrettyPrinter;
 import com.fasterxml.jackson.core.util.Instantiatable;
 import com.fasterxml.jackson.core.util.MinimalPrettyPrinter;
-
+import com.fasterxml.jackson.databind.cfg.ContextAttributes;
 import com.fasterxml.jackson.databind.jsonFormatVisitors.JsonFormatVisitorWrapper;
 import com.fasterxml.jackson.databind.ser.DefaultSerializerProvider;
 import com.fasterxml.jackson.databind.ser.FilterProvider;
@@ -31,7 +34,7 @@ public class ObjectWriter
     implements Versioned,
         java.io.Serializable // since 2.1
 {
-    private static final long serialVersionUID = -7024829992408267532L;
+    private static final long serialVersionUID = -7040667122552707164L;
 
     /**
      * We need to keep track of explicit disabling of pretty printing;
@@ -57,11 +60,11 @@ public class ObjectWriter
     /**
      * Factory used for constructing {@link JsonGenerator}s
      */
-    protected final JsonFactory _jsonFactory;
-    
+    protected final JsonFactory _generatorFactory;
+
     /*
     /**********************************************************
-    /* Configuration that can be changed during building
+    /* Configuration that can be changed via mutant factories
     /**********************************************************
      */
 
@@ -94,6 +97,25 @@ public class ObjectWriter
      */
     protected final FormatSchema _schema;
     
+    /**
+     * Caller may want to specify character escaping details, either as
+     * defaults, or on call-by-call basis.
+     * 
+     * @since 2.3
+     */
+    protected final CharacterEscapes _characterEscapes;
+
+    /*
+    /**********************************************************
+    /* Derived settings
+    /**********************************************************
+     */
+
+    /**
+     * @since 2.3
+     */
+    protected final boolean  _cfgBigDecimalAsPlain;
+
     /*
     /**********************************************************
     /* Life-cycle, constructors
@@ -107,19 +129,23 @@ public class ObjectWriter
             JavaType rootType, PrettyPrinter pp)
     {
         _config = config;
+        _cfgBigDecimalAsPlain = _config.isEnabled(SerializationFeature.WRITE_BIGDECIMAL_AS_PLAIN);
 
         _serializerProvider = mapper._serializerProvider;
         _serializerFactory = mapper._serializerFactory;
-        _jsonFactory = mapper._jsonFactory;
-
-        if (rootType != null) {
-            rootType = rootType.withStaticTyping();
-        }
-        _rootType = rootType;
+        _generatorFactory = mapper._jsonFactory;
         _prettyPrinter = pp;
         _schema = null;
+        _characterEscapes = null;
 
-        _rootSerializer = _prefetchRootSerializer(config, rootType);
+        // 29-Apr-2014, tatu: There is no "untyped serializer", so:
+        if (rootType == null || rootType.hasRawClass(Object.class)) {
+            _rootType = null;
+            _rootSerializer = null;
+        } else {
+            _rootType = rootType.withStaticTyping();
+            _rootSerializer = _prefetchRootSerializer(config, _rootType);
+        }
     }
 
     /**
@@ -128,15 +154,17 @@ public class ObjectWriter
     protected ObjectWriter(ObjectMapper mapper, SerializationConfig config)
     {
         _config = config;
+        _cfgBigDecimalAsPlain = _config.isEnabled(SerializationFeature.WRITE_BIGDECIMAL_AS_PLAIN);
 
         _serializerProvider = mapper._serializerProvider;
         _serializerFactory = mapper._serializerFactory;
-        _jsonFactory = mapper._jsonFactory;
+        _generatorFactory = mapper._jsonFactory;
 
         _rootType = null;
         _rootSerializer = null;
         _prettyPrinter = null;
         _schema = null;
+        _characterEscapes = null;
     }
 
     /**
@@ -146,15 +174,17 @@ public class ObjectWriter
             FormatSchema s)
     {
         _config = config;
+        _cfgBigDecimalAsPlain = _config.isEnabled(SerializationFeature.WRITE_BIGDECIMAL_AS_PLAIN);
 
         _serializerProvider = mapper._serializerProvider;
         _serializerFactory = mapper._serializerFactory;
-        _jsonFactory = mapper._jsonFactory;
+        _generatorFactory = mapper._jsonFactory;
 
         _rootType = null;
         _rootSerializer = null;
         _prettyPrinter = null;
         _schema = s;
+        _characterEscapes = null;
     }
     
     /**
@@ -162,18 +192,20 @@ public class ObjectWriter
      */
     protected ObjectWriter(ObjectWriter base, SerializationConfig config,
             JavaType rootType, JsonSerializer<Object> rootSer,
-            PrettyPrinter pp, FormatSchema s)
+            PrettyPrinter pp, FormatSchema s, CharacterEscapes escapes)
     {
         _config = config;
+        _cfgBigDecimalAsPlain = _config.isEnabled(SerializationFeature.WRITE_BIGDECIMAL_AS_PLAIN);
 
         _serializerProvider = base._serializerProvider;
         _serializerFactory = base._serializerFactory;
-        _jsonFactory = base._jsonFactory;
+        _generatorFactory = base._generatorFactory;
 
         _rootType = rootType;
         _rootSerializer = rootSer;
         _prettyPrinter = pp;
         _schema = s;
+        _characterEscapes = escapes;
     }
 
     /**
@@ -182,11 +214,34 @@ public class ObjectWriter
     protected ObjectWriter(ObjectWriter base, SerializationConfig config)
     {
         _config = config;
+        _cfgBigDecimalAsPlain = _config.isEnabled(SerializationFeature.WRITE_BIGDECIMAL_AS_PLAIN);
 
         _serializerProvider = base._serializerProvider;
         _serializerFactory = base._serializerFactory;
-        _jsonFactory = base._jsonFactory;
+        _generatorFactory = base._generatorFactory;
         _schema = base._schema;
+        _characterEscapes = base._characterEscapes;
+
+        _rootType = base._rootType;
+        _rootSerializer = base._rootSerializer;
+        _prettyPrinter = base._prettyPrinter;
+    }
+
+    /**
+     * @since 2.3
+     */
+    protected ObjectWriter(ObjectWriter base, JsonFactory f)
+    {
+        // may need to override ordering, based on data format capabilities
+        _config = base._config
+            .with(MapperFeature.SORT_PROPERTIES_ALPHABETICALLY, f.requiresPropertyOrdering());
+        _cfgBigDecimalAsPlain = base._cfgBigDecimalAsPlain;
+
+        _serializerProvider = base._serializerProvider;
+        _serializerFactory = base._serializerFactory;
+        _generatorFactory = base._generatorFactory;
+        _schema = base._schema;
+        _characterEscapes = base._characterEscapes;
 
         _rootType = base._rootType;
         _rootSerializer = base._rootSerializer;
@@ -212,8 +267,7 @@ public class ObjectWriter
      * Method for constructing a new instance that is configured
      * with specified feature enabled.
      */
-    public ObjectWriter with(SerializationFeature feature) 
-    {
+    public ObjectWriter with(SerializationFeature feature)  {
         SerializationConfig newConfig = _config.with(feature);
         return (newConfig == _config) ? this : new ObjectWriter(this, newConfig);
     }
@@ -222,9 +276,7 @@ public class ObjectWriter
      * Method for constructing a new instance that is configured
      * with specified features enabled.
      */
-    public ObjectWriter with(SerializationFeature first,
-            SerializationFeature... other)
-    {
+    public ObjectWriter with(SerializationFeature first, SerializationFeature... other) {
         SerializationConfig newConfig = _config.with(first, other);
         return (newConfig == _config) ? this : new ObjectWriter(this, newConfig);
     }    
@@ -233,8 +285,7 @@ public class ObjectWriter
      * Method for constructing a new instance that is configured
      * with specified features enabled.
      */
-    public ObjectWriter withFeatures(SerializationFeature... features)
-    {
+    public ObjectWriter withFeatures(SerializationFeature... features) {
         SerializationConfig newConfig = _config.withFeatures(features);
         return (newConfig == _config) ? this : new ObjectWriter(this, newConfig);
     }    
@@ -243,8 +294,7 @@ public class ObjectWriter
      * Method for constructing a new instance that is configured
      * with specified feature enabled.
      */
-    public ObjectWriter without(SerializationFeature feature) 
-    {
+    public ObjectWriter without(SerializationFeature feature) {
         SerializationConfig newConfig = _config.without(feature);
         return (newConfig == _config) ? this : new ObjectWriter(this, newConfig);
     }    
@@ -253,9 +303,7 @@ public class ObjectWriter
      * Method for constructing a new instance that is configured
      * with specified features enabled.
      */
-    public ObjectWriter without(SerializationFeature first,
-            SerializationFeature... other)
-    {
+    public ObjectWriter without(SerializationFeature first, SerializationFeature... other) {
         SerializationConfig newConfig = _config.without(first, other);
         return (newConfig == _config) ? this : new ObjectWriter(this, newConfig);
     }    
@@ -264,8 +312,7 @@ public class ObjectWriter
      * Method for constructing a new instance that is configured
      * with specified features enabled.
      */
-    public ObjectWriter withoutFeatures(SerializationFeature... features)
-    {
+    public ObjectWriter withoutFeatures(SerializationFeature... features) {
         SerializationConfig newConfig = _config.withoutFeatures(features);
         return (newConfig == _config) ? this : new ObjectWriter(this, newConfig);
     }    
@@ -278,8 +325,7 @@ public class ObjectWriter
      * Note that the method does NOT change state of this reader, but
      * rather construct and returns a newly configured instance.
      */
-    public ObjectWriter with(DateFormat df)
-    {
+    public ObjectWriter with(DateFormat df) {
         SerializationConfig newConfig = _config.with(df);
         return (newConfig == _config) ? this : new ObjectWriter(this, newConfig);
     }
@@ -288,8 +334,7 @@ public class ObjectWriter
      * Method that will construct a new instance that will use the default
      * pretty printer for serialization.
      */
-    public ObjectWriter withDefaultPrettyPrinter()
-    {
+    public ObjectWriter withDefaultPrettyPrinter() {
         return with(new DefaultPrettyPrinter());
     }
 
@@ -297,20 +342,16 @@ public class ObjectWriter
      * Method that will construct a new instance that uses specified
      * provider for resolving filter instances by id.
      */
-    public ObjectWriter with(FilterProvider filterProvider)
-    {
-        if (filterProvider == _config.getFilterProvider()) { // no change?
-            return this;
-        }
-        return new ObjectWriter(this, _config.withFilters(filterProvider));
+    public ObjectWriter with(FilterProvider filterProvider) {
+        return (filterProvider == _config.getFilterProvider()) ? this
+                 : new ObjectWriter(this, _config.withFilters(filterProvider));
     }
 
     /**
      * Method that will construct a new instance that will use specified pretty
      * printer (or, if null, will not do any pretty-printing)
      */
-    public ObjectWriter with(PrettyPrinter pp)
-    {
+    public ObjectWriter with(PrettyPrinter pp) {
         if (pp == _prettyPrinter) {
             return this;
         }
@@ -318,7 +359,8 @@ public class ObjectWriter
         if (pp == null) {
             pp = NULL_PRETTY_PRINTER;
         }
-        return new ObjectWriter(this, _config, _rootType, _rootSerializer, pp, _schema);
+        return new ObjectWriter(this, _config, _rootType, _rootSerializer,
+                pp, _schema, _characterEscapes);
     }
 
     /**
@@ -329,8 +371,7 @@ public class ObjectWriter
      * Note that method does NOT change state of this reader, but
      * rather construct and returns a newly configured instance.
      */
-    public ObjectWriter withRootName(String rootName)
-    {
+    public ObjectWriter withRootName(String rootName) {
         SerializationConfig newConfig = _config.withRootName(rootName);
         return (newConfig == _config) ? this :  new ObjectWriter(this, newConfig);
     }
@@ -343,13 +384,13 @@ public class ObjectWriter
      * rather construct and returns a newly configured instance.
      */
     
-    public ObjectWriter withSchema(FormatSchema schema)
-    {
+    public ObjectWriter withSchema(FormatSchema schema) {
         if (_schema == schema) {
             return this;
         }
         _verifySchemaType(schema);
-        return new ObjectWriter(this, _config, _rootType, _rootSerializer, _prettyPrinter, schema);
+        return new ObjectWriter(this, _config, _rootType, _rootSerializer,
+                _prettyPrinter, schema, _characterEscapes);
     }
 
     /**
@@ -362,10 +403,17 @@ public class ObjectWriter
      */
     public ObjectWriter withType(JavaType rootType)
     {
-        // 15-Mar-2013, tatu: Important! Indicate that static typing is needed:
-        rootType = rootType.withStaticTyping();
-        JsonSerializer<Object> rootSer = _prefetchRootSerializer(_config, rootType);
-        return new ObjectWriter(this, _config, rootType, rootSer, _prettyPrinter, _schema);
+        JsonSerializer<Object> rootSer;
+        if (rootType == null || rootType.hasRawClass(Object.class)) {
+            rootType = null;
+            rootSer = null;
+        } else {
+            // 15-Mar-2013, tatu: Important! Indicate that static typing is needed:
+            rootType = rootType.withStaticTyping();
+            rootSer = _prefetchRootSerializer(_config, rootType);
+        }
+        return new ObjectWriter(this, _config, rootType, rootSer,
+                _prettyPrinter, _schema, _characterEscapes);
     }    
 
     /**
@@ -374,6 +422,9 @@ public class ObjectWriter
      * type of the root object itself.
      */
     public ObjectWriter withType(Class<?> rootType) {
+        if (rootType == Object.class) {
+            return withType((JavaType) null);
+        }
         return withType(_config.constructType(rootType));
     }
 
@@ -414,6 +465,56 @@ public class ObjectWriter
         SerializationConfig newConfig = _config.with(b64variant);
         return (newConfig == _config) ? this :  new ObjectWriter(this, newConfig);
     }
+
+    /**
+     * @since 2.3
+     */
+    public ObjectWriter with(CharacterEscapes escapes) {
+        if (_characterEscapes == escapes) {
+            return this;
+        }
+        return new ObjectWriter(this, _config, _rootType, _rootSerializer,
+                _prettyPrinter, _schema, escapes);
+    }
+
+    /**
+     * @since 2.3
+     */
+    public ObjectWriter with(JsonFactory f) {
+        return (f == _generatorFactory) ? this : new ObjectWriter(this, f);
+    }    
+
+    /**
+     * @since 2.3
+     */
+    public ObjectWriter with(ContextAttributes attrs) {
+        SerializationConfig newConfig = _config.with(attrs);
+        return (newConfig == _config) ? this :  new ObjectWriter(this, newConfig);
+    }
+
+    /**
+     * @since 2.3
+     */
+    public ObjectWriter withAttributes(Map<Object,Object> attrs) {
+        SerializationConfig newConfig = _config.withAttributes(attrs);
+        return (newConfig == _config) ? this :  new ObjectWriter(this, newConfig);
+    }
+
+    /**
+     * @since 2.3
+     */
+    public ObjectWriter withAttribute(Object key, Object value) {
+        SerializationConfig newConfig = _config.withAttribute(key, value);
+        return (newConfig == _config) ? this :  new ObjectWriter(this, newConfig);
+    }
+
+    /**
+     * @since 2.3
+     */
+    public ObjectWriter withoutAttribute(Object key) {
+        SerializationConfig newConfig = _config.withoutAttribute(key);
+        return (newConfig == _config) ? this :  new ObjectWriter(this, newConfig);
+    }
     
     /*
     /**********************************************************
@@ -430,7 +531,7 @@ public class ObjectWriter
     }
 
     public boolean isEnabled(JsonParser.Feature f) {
-        return _jsonFactory.isEnabled(f);
+        return _generatorFactory.isEnabled(f);
     }
 
     /**
@@ -445,14 +546,14 @@ public class ObjectWriter
      */
     @Deprecated
     public JsonFactory getJsonFactory() {
-        return _jsonFactory;
+        return _generatorFactory;
     }
 
     /**
      * @since 2.2
      */
     public JsonFactory getFactory() {
-        return _jsonFactory;
+        return _generatorFactory;
     }
     
     public TypeFactory getTypeFactory() {
@@ -470,6 +571,13 @@ public class ObjectWriter
     public boolean hasPrefetchedSerializer() {
         return _rootSerializer != null;
     }
+
+    /**
+     * @since 2.3
+     */
+    public ContextAttributes getAttributes() {
+        return _config.getAttributes();
+    }
     
     /*
     /**********************************************************
@@ -514,7 +622,7 @@ public class ObjectWriter
     public void writeValue(File resultFile, Object value)
         throws IOException, JsonGenerationException, JsonMappingException
     {
-        _configAndWriteValue(_jsonFactory.createGenerator(resultFile, JsonEncoding.UTF8), value);
+        _configAndWriteValue(_generatorFactory.createGenerator(resultFile, JsonEncoding.UTF8), value);
     }
 
     /**
@@ -531,7 +639,7 @@ public class ObjectWriter
     public void writeValue(OutputStream out, Object value)
         throws IOException, JsonGenerationException, JsonMappingException
     {
-        _configAndWriteValue(_jsonFactory.createGenerator(out, JsonEncoding.UTF8), value);
+        _configAndWriteValue(_generatorFactory.createGenerator(out, JsonEncoding.UTF8), value);
     }
 
     /**
@@ -547,7 +655,7 @@ public class ObjectWriter
     public void writeValue(Writer w, Object value)
         throws IOException, JsonGenerationException, JsonMappingException
     {
-        _configAndWriteValue(_jsonFactory.createGenerator(w), value);
+        _configAndWriteValue(_generatorFactory.createGenerator(w), value);
     }
 
     /**
@@ -558,13 +666,14 @@ public class ObjectWriter
      *<p>
      * Note: prior to version 2.1, throws clause included {@link IOException}; 2.1 removed it.
      */
+    @SuppressWarnings("resource")
     public String writeValueAsString(Object value)
         throws JsonProcessingException
     {        
         // alas, we have to pull the recycler directly here...
-        SegmentedStringWriter sw = new SegmentedStringWriter(_jsonFactory._getBufferRecycler());
+        SegmentedStringWriter sw = new SegmentedStringWriter(_generatorFactory._getBufferRecycler());
         try {
-            _configAndWriteValue(_jsonFactory.createGenerator(sw), value);
+            _configAndWriteValue(_generatorFactory.createGenerator(sw), value);
         } catch (JsonProcessingException e) { // to support [JACKSON-758]
             throw e;
         } catch (IOException e) { // shouldn't really happen, but is declared as possibility so:
@@ -582,12 +691,13 @@ public class ObjectWriter
      *<p>
      * Note: prior to version 2.1, throws clause included {@link IOException}; 2.1 removed it.
      */
+    @SuppressWarnings("resource")
     public byte[] writeValueAsBytes(Object value)
         throws JsonProcessingException
     {
-        ByteArrayBuilder bb = new ByteArrayBuilder(_jsonFactory._getBufferRecycler());
+        ByteArrayBuilder bb = new ByteArrayBuilder(_generatorFactory._getBufferRecycler());
         try {
-            _configAndWriteValue(_jsonFactory.createGenerator(bb, JsonEncoding.UTF8), value);
+            _configAndWriteValue(_generatorFactory.createGenerator(bb, JsonEncoding.UTF8), value);
         } catch (JsonProcessingException e) { // to support [JACKSON-758]
             throw e;
         } catch (IOException e) { // shouldn't really happen, but is declared as possibility so:
@@ -626,9 +736,19 @@ public class ObjectWriter
     }
     
     public boolean canSerialize(Class<?> type) {
-        return _serializerProvider(_config).hasSerializerFor(type);
+        return _serializerProvider(_config).hasSerializerFor(type, null);
     }
 
+    /**
+     * Method for checking whether instances of given type can be serialized,
+     * and optionally why (as per {@link Throwable} returned).
+     * 
+     * @since 2.3
+     */
+    public boolean canSerialize(Class<?> type, AtomicReference<Throwable> cause) {
+        return _serializerProvider(_config).hasSerializerFor(type, cause);
+    }
+    
     /*
     /**********************************************************
     /* Overridable helper methods
@@ -655,9 +775,9 @@ public class ObjectWriter
     protected void _verifySchemaType(FormatSchema schema)
     {
         if (schema != null) {
-            if (!_jsonFactory.canUseSchema(schema)) {
+            if (!_generatorFactory.canUseSchema(schema)) {
                     throw new IllegalArgumentException("Can not use FormatSchema of type "+schema.getClass().getName()
-                            +" for format "+_jsonFactory.getFormatName());
+                            +" for format "+_generatorFactory.getFormatName());
             }
         }
     }
@@ -666,8 +786,7 @@ public class ObjectWriter
      * Method called to configure the generator as necessary and then
      * call write functionality
      */
-    protected final void _configAndWriteValue(JsonGenerator jgen, Object value)
-        throws IOException, JsonGenerationException, JsonMappingException
+    protected final void _configAndWriteValue(JsonGenerator jgen, Object value) throws IOException
     {
         _configureJsonGenerator(jgen);
         // [JACKSON-282]: consider Closeable
@@ -689,6 +808,10 @@ public class ObjectWriter
              * will not mask exception that is pending)
              */
             if (!closed) {
+                /* 04-Mar-2014, tatu: But! Let's try to prevent auto-closing of
+                 *    structures, which typically causes more damage.
+                 */
+                jgen.disable(JsonGenerator.Feature.AUTO_CLOSE_JSON_CONTENT);
                 try {
                     jgen.close();
                 } catch (IOException ioe) { }
@@ -721,6 +844,10 @@ public class ObjectWriter
              * been closed
              */
             if (jgen != null) {
+                /* 04-Mar-2014, tatu: But! Let's try to prevent auto-closing of
+                 *    structures, which typically causes more damage.
+                 */
+                jgen.disable(JsonGenerator.Feature.AUTO_CLOSE_JSON_CONTENT);
                 try {
                     jgen.close();
                 } catch (IOException ioe) { }
@@ -805,9 +932,15 @@ public class ObjectWriter
         } else if (_config.isEnabled(SerializationFeature.INDENT_OUTPUT)) {
             jgen.useDefaultPrettyPrinter();
         }
+        if (_characterEscapes != null) {
+            jgen.setCharacterEscapes(_characterEscapes);
+        }
         // [JACKSON-520]: add support for pass-through schema:
         if (_schema != null) {
             jgen.setSchema(_schema);
         }
+        if (_cfgBigDecimalAsPlain) { // should only set if explicitly set; this should work for now:
+            jgen.enable(JsonGenerator.Feature.WRITE_BIGDECIMAL_AS_PLAIN);
+        }
     }
 }
