@@ -7,6 +7,7 @@ import com.fasterxml.jackson.annotation.*;
 import com.fasterxml.jackson.annotation.JsonAutoDetect.Visibility;
 import com.fasterxml.jackson.core.Base64Variant;
 import com.fasterxml.jackson.databind.cfg.BaseSettings;
+import com.fasterxml.jackson.databind.cfg.ContextAttributes;
 import com.fasterxml.jackson.databind.cfg.HandlerInstantiator;
 import com.fasterxml.jackson.databind.cfg.MapperConfigBase;
 import com.fasterxml.jackson.databind.introspect.ClassIntrospector;
@@ -36,8 +37,8 @@ public final class SerializationConfig
     extends MapperConfigBase<SerializationFeature, SerializationConfig>
     implements java.io.Serializable // since 2.1
 {
-    // for 2.1.0:
-    private static final long serialVersionUID = 8849092838541724233L;
+    // Valid as of 2.4
+    private static final long serialVersionUID = -1278867172535832879L;
 
     /**
      * Set of features enabled; actual type (kind of features)
@@ -84,8 +85,7 @@ public final class SerializationConfig
         _filterProvider = src._filterProvider;
     }
 
-    private SerializationConfig(SerializationConfig src,
-            int mapperFeatures, int serFeatures)
+    private SerializationConfig(SerializationConfig src, int mapperFeatures, int serFeatures)
     {
         super(src, mapperFeatures);
         _serFeatures = serFeatures;
@@ -143,6 +143,17 @@ public final class SerializationConfig
         _serializationInclusion = src._serializationInclusion;
         _filterProvider = src._filterProvider;
     }
+
+    /**
+     * @since 2.1
+     */
+    protected SerializationConfig(SerializationConfig src, ContextAttributes attrs)
+    {
+        super(src, attrs);
+        _serFeatures = src._serFeatures;
+        _serializationInclusion = src._serializationInclusion;
+        _filterProvider = src._filterProvider;
+    }
     
     /*
     /**********************************************************
@@ -180,6 +191,19 @@ public final class SerializationConfig
                 : new SerializationConfig(this, newMapperFlags, _serFeatures);
     }
 
+    @Override
+    public SerializationConfig with(MapperFeature feature, boolean state)
+    {
+        int newMapperFlags;
+        if (state) {
+            newMapperFlags = _mapperFeatures | feature.getMask();
+        } else {
+            newMapperFlags = _mapperFeatures & ~feature.getMask();
+        }
+        return (newMapperFlags == _mapperFeatures) ? this :
+            new SerializationConfig(this, newMapperFlags, _serFeatures);
+    }
+    
     @Override
     public SerializationConfig with(AnnotationIntrospector ai) {
         return _withBase(_base.withAnnotationIntrospector(ai));
@@ -283,6 +307,11 @@ public final class SerializationConfig
     public SerializationConfig with(Base64Variant base64) {
         return _withBase(_base.with(base64));
     }
+
+    @Override
+    public SerializationConfig with(ContextAttributes attrs) {
+        return (attrs == _attributes) ? this : new SerializationConfig(this, attrs);
+    }
     
     private final SerializationConfig _withBase(BaseSettings newBase) {
         return (_base == newBase) ? this : new SerializationConfig(this, newBase);
@@ -452,7 +481,17 @@ public final class SerializationConfig
     public final boolean isEnabled(SerializationFeature f) {
         return (_serFeatures & f.getMask()) != 0;
     }
-    
+
+    /**
+     * "Bulk" access method for checking that all features specified by
+     * mask are enabled.
+     * 
+     * @since 2.3
+     */
+    public final boolean hasSerializationFeatures(int featureMask) {
+        return (_serFeatures & featureMask) == featureMask;
+    }
+
     public final int getSerializationFeatures() {
         return _serFeatures;
     }
@@ -474,7 +513,7 @@ public final class SerializationConfig
     public FilterProvider getFilterProvider() {
         return _filterProvider;
     }
-
+    
     /*
     /**********************************************************
     /* Introspection methods
