@@ -103,9 +103,12 @@ public class BeanDeserializer
 
     /**
      * Main deserialization method for bean-based objects (POJOs).
+     *<p>
+     * NOTE: was declared 'final' in 2.2; should NOT be to let extensions
+     * like Afterburner change definition.
      */
     @Override
-    public final Object deserialize(JsonParser jp, DeserializationContext ctxt)
+    public Object deserialize(JsonParser jp, DeserializationContext ctxt)
         throws IOException, JsonProcessingException
     {
         JsonToken t = jp.getCurrentToken();
@@ -123,13 +126,9 @@ public class BeanDeserializer
         return _deserializeOther(jp, ctxt, t);
     }
 
-    private final Object _deserializeOther(JsonParser jp, DeserializationContext ctxt,
-            JsonToken t)
-        throws IOException, JsonProcessingException
+    protected final Object _deserializeOther(JsonParser jp, DeserializationContext ctxt,
+            JsonToken t) throws IOException
     {
-        if (t == null) {
-            return _missingToken(jp, ctxt);
-        }
         // and then others, generally requiring use of @JsonCreator
         switch (t) {
         case VALUE_STRING:
@@ -139,7 +138,7 @@ public class BeanDeserializer
         case VALUE_NUMBER_FLOAT:
 	    return deserializeFromDouble(jp, ctxt);
         case VALUE_EMBEDDED_OBJECT:
-            return jp.getEmbeddedObject();
+            return deserializeFromEmbedded(jp, ctxt);
         case VALUE_TRUE:
         case VALUE_FALSE:
             return deserializeFromBoolean(jp, ctxt);
@@ -156,14 +155,14 @@ public class BeanDeserializer
             }
             return deserializeFromObject(jp, ctxt);
         default:
-            throw ctxt.mappingException(getBeanClass());
+            throw ctxt.mappingException(handledType());
         }
     }
 
     protected Object _missingToken(JsonParser jp, DeserializationContext ctxt)
         throws JsonProcessingException
     {
-        throw ctxt.endOfInputException(getBeanClass());
+        throw ctxt.endOfInputException(handledType());
     }
     
     /**
@@ -209,22 +208,11 @@ public class BeanDeserializer
                 }
                 continue;
             }
-            /* As per [JACKSON-313], things marked as ignorable should not be
-             * passed to any setter
-             */
-            if (_ignorableProps != null && _ignorableProps.contains(propName)) {
-                jp.skipChildren();
-            } else if (_anySetter != null) {
-                _anySetter.deserializeAndSet(jp, ctxt, bean, propName);
-                continue;
-            } else {
-                // Unknown: let's call handler method
-                handleUnknownProperty(jp, ctxt, bean, propName);
-            }
+            handleUnknownVanilla(jp, ctxt, bean, propName);
         }
         return bean;
     }
-    
+
     /*
     /**********************************************************
     /* Concrete deserialization methods
@@ -240,7 +228,7 @@ public class BeanDeserializer
         throws IOException, JsonProcessingException
     {
         final Object bean = _valueInstantiator.createUsingDefault(ctxt);
-        for (; jp.getCurrentToken() != JsonToken.END_OBJECT; jp.nextToken()) {
+        for (; t == JsonToken.FIELD_NAME; t = jp.nextToken()) {
             String propName = jp.getCurrentName();
             // Skip field name:
             jp.nextToken();
@@ -273,9 +261,31 @@ public class BeanDeserializer
             if (_externalTypeIdHandler != null) {
                 return deserializeWithExternalTypeId(jp, ctxt);
             }
-            return deserializeFromObjectUsingNonDefault(jp, ctxt);
+            Object bean = deserializeFromObjectUsingNonDefault(jp, ctxt);
+            if (_injectables != null) {
+                injectValues(ctxt, bean);
+            }
+            /* 27-May-2014, tatu: I don't think view processing would work
+             *   at this point, so commenting it out; but leaving in place
+             *   just in case I forgot something fundamental...
+             */
+            /*
+            if (_needViewProcesing) {
+                Class<?> view = ctxt.getActiveView();
+                if (view != null) {
+                    return deserializeWithView(jp, ctxt, bean, view);
+                }
+            }
+            */
+            return bean;
         }
         final Object bean = _valueInstantiator.createUsingDefault(ctxt);
+        if (jp.canReadObjectId()) {
+            Object id = jp.getObjectId();
+            if (id != null) {
+                _handleTypedObjectId(jp, ctxt, bean, id);
+            }
+        }
         if (_injectables != null) {
             injectValues(ctxt, bean);
         }
@@ -285,7 +295,8 @@ public class BeanDeserializer
                 return deserializeWithView(jp, ctxt, bean, view);
             }
         }
-        for (; jp.getCurrentToken() != JsonToken.END_OBJECT; jp.nextToken()) {
+        JsonToken t = jp.getCurrentToken();
+        for (; t == JsonToken.FIELD_NAME; t = jp.nextToken()) {
             String propName = jp.getCurrentName();
             // Skip field name:
             jp.nextToken();
@@ -298,22 +309,7 @@ public class BeanDeserializer
                 }
                 continue;
             }
-            /* As per [JACKSON-313], things marked as ignorable should not be
-             * passed to any setter
-             */
-            if (_ignorableProps != null && _ignorableProps.contains(propName)) {
-                jp.skipChildren();
-            } else if (_anySetter != null) {
-                try {
-                    _anySetter.deserializeAndSet(jp, ctxt, bean, propName);
-                } catch (Exception e) {
-                    wrapAndThrow(e, bean, propName, ctxt);
-                }
-                continue;
-            } else {
-                // Unknown: let's call handler method
-                handleUnknownProperty(jp, ctxt, bean, propName);         
-            }
+            handleUnknownVanilla(jp, ctxt, bean, propName);
         }
         return bean;
     }
@@ -327,6 +323,7 @@ public class BeanDeserializer
      * as well.
      */
     @Override
+    @SuppressWarnings("resource")
     protected Object _deserializeUsingPropertyBased(final JsonParser jp, final DeserializationContext ctxt)
         throws IOException, JsonProcessingException
     { 
@@ -379,7 +376,7 @@ public class BeanDeserializer
             // As per [JACKSON-313], things marked as ignorable should not be
             // passed to any setter
             if (_ignorableProps != null && _ignorableProps.contains(propName)) {
-                jp.skipChildren();
+                handleIgnoredProperty(jp, ctxt, handledType(), propName);
                 continue;
             }
             // "any property"?
@@ -389,7 +386,7 @@ public class BeanDeserializer
             }
             // Ok then, let's collect the whole field; name and value
             if (unknown == null) {
-                unknown = new TokenBuffer(jp.getCodec());
+                unknown = new TokenBuffer(jp);
             }
             unknown.writeFieldName(propName);
             unknown.copyCurrentStructure(jp);
@@ -442,18 +439,7 @@ public class BeanDeserializer
                 }
                 continue;
             }
-            /* As per [JACKSON-313], things marked as ignorable should not be
-             * passed to any setter
-             */
-            if (_ignorableProps != null && _ignorableProps.contains(propName)) {
-                jp.skipChildren();
-            } else if (_anySetter != null) {
-                _anySetter.deserializeAndSet(jp, ctxt, bean, propName);
-                continue;
-            } else {
-                // Unknown: let's call handler method
-                handleUnknownProperty(jp, ctxt, bean, propName);
-            }
+            handleUnknownVanilla(jp, ctxt, bean, propName);
         }
         return bean;
     }
@@ -468,6 +454,7 @@ public class BeanDeserializer
      * Method called when there are declared "unwrapped" properties
      * which need special handling
      */
+    @SuppressWarnings("resource")
     protected Object deserializeWithUnwrapped(JsonParser jp, DeserializationContext ctxt)
         throws IOException, JsonProcessingException
     {
@@ -477,7 +464,7 @@ public class BeanDeserializer
         if (_propertyBasedCreator != null) {
             return deserializeUsingPropertyBasedWithUnwrapped(jp, ctxt);
         }
-        TokenBuffer tokens = new TokenBuffer(jp.getCodec());
+        TokenBuffer tokens = new TokenBuffer(jp);
         tokens.writeStartObject();
         final Object bean = _valueInstantiator.createUsingDefault(ctxt);
 
@@ -503,7 +490,7 @@ public class BeanDeserializer
             }
             // ignorable things should be ignored
             if (_ignorableProps != null && _ignorableProps.contains(propName)) {
-                jp.skipChildren();
+                handleIgnoredProperty(jp, ctxt, bean, propName);
                 continue;
             }
             // but... others should be passed to unwrapped property deserializers
@@ -524,6 +511,7 @@ public class BeanDeserializer
         return bean;
     }    
 
+    @SuppressWarnings("resource")
     protected Object deserializeWithUnwrapped(JsonParser jp, DeserializationContext ctxt, Object bean)
         throws IOException, JsonProcessingException
     {
@@ -531,7 +519,7 @@ public class BeanDeserializer
         if (t == JsonToken.START_OBJECT) {
             t = jp.nextToken();
         }
-        TokenBuffer tokens = new TokenBuffer(jp.getCodec());
+        TokenBuffer tokens = new TokenBuffer(jp);
         tokens.writeStartObject();
         final Class<?> activeView = _needViewProcesing ? ctxt.getActiveView() : null;
         for (; t == JsonToken.FIELD_NAME; t = jp.nextToken()) {
@@ -551,7 +539,7 @@ public class BeanDeserializer
                 continue;
             }
             if (_ignorableProps != null && _ignorableProps.contains(propName)) {
-                jp.skipChildren();
+                handleIgnoredProperty(jp, ctxt, bean, propName);
                 continue;
             }
             // but... others should be passed to unwrapped property deserializers
@@ -574,7 +562,7 @@ public class BeanDeserializer
         final PropertyBasedCreator creator = _propertyBasedCreator;
         PropertyValueBuffer buffer = creator.startBuilding(jp, ctxt, _objectIdReader);
 
-        TokenBuffer tokens = new TokenBuffer(jp.getCodec());
+        TokenBuffer tokens = new TokenBuffer(jp);
         tokens.writeStartObject();
 
         JsonToken t = jp.getCurrentToken();
@@ -626,7 +614,7 @@ public class BeanDeserializer
              * passed to any setter
              */
             if (_ignorableProps != null && _ignorableProps.contains(propName)) {
-                jp.skipChildren();
+                handleIgnoredProperty(jp, ctxt, handledType(), propName);
                 continue;
             }
             tokens.writeFieldName(propName);
@@ -670,7 +658,8 @@ public class BeanDeserializer
     {
         final Class<?> activeView = _needViewProcesing ? ctxt.getActiveView() : null;
         final ExternalTypeHandler ext = _externalTypeIdHandler.start();
-        for (; jp.getCurrentToken() != JsonToken.END_OBJECT; jp.nextToken()) {
+        JsonToken t = jp.getCurrentToken();
+        for (; t == JsonToken.FIELD_NAME; t = jp.nextToken()) {
             String propName = jp.getCurrentName();
             jp.nextToken();
             SettableBeanProperty prop = _beanProperties.find(propName);
@@ -692,7 +681,7 @@ public class BeanDeserializer
             }
             // ignorable things should be ignored
             if (_ignorableProps != null && _ignorableProps.contains(propName)) {
-                jp.skipChildren();
+                handleIgnoredProperty(jp, ctxt, bean, propName);
                 continue;
             }
             // but others are likely to be part of external type id thingy...
@@ -723,7 +712,7 @@ public class BeanDeserializer
         final PropertyBasedCreator creator = _propertyBasedCreator;
         PropertyValueBuffer buffer = creator.startBuilding(jp, ctxt, _objectIdReader);
 
-        TokenBuffer tokens = new TokenBuffer(jp.getCodec());
+        TokenBuffer tokens = new TokenBuffer(jp);
         tokens.writeStartObject();
 
         JsonToken t = jp.getCurrentToken();
@@ -782,7 +771,7 @@ public class BeanDeserializer
              * passed to any setter
              */
             if (_ignorableProps != null && _ignorableProps.contains(propName)) {
-                jp.skipChildren();
+                handleIgnoredProperty(jp, ctxt, handledType(), propName);
                 continue;
             }
             // "any property"?
