@@ -6,12 +6,12 @@ import java.util.concurrent.atomic.AtomicReference;
 
 import com.fasterxml.jackson.annotation.ObjectIdGenerator;
 import com.fasterxml.jackson.annotation.ObjectIdGenerators;
-
+import com.fasterxml.jackson.annotation.ObjectIdResolver;
 import com.fasterxml.jackson.databind.*;
 import com.fasterxml.jackson.databind.annotation.JsonPOJOBuilder;
 import com.fasterxml.jackson.databind.cfg.DeserializerFactoryConfig;
 import com.fasterxml.jackson.databind.deser.impl.*;
-import com.fasterxml.jackson.databind.deser.std.JdkDeserializers;
+import com.fasterxml.jackson.databind.deser.std.AtomicReferenceDeserializer;
 import com.fasterxml.jackson.databind.deser.std.ThrowableDeserializer;
 import com.fasterxml.jackson.databind.ext.OptionalHandlerFactory;
 import com.fasterxml.jackson.databind.introspect.*;
@@ -207,7 +207,10 @@ public class BeanDeserializerFactory
             } else {
                 referencedType = params[0];
             }
-            return new JdkDeserializers.AtomicReferenceDeserializer(referencedType);
+            TypeDeserializer valueTypeDeser = findTypeDeserializer(ctxt.getConfig(), referencedType);
+            BeanDescription refdDesc = ctxt.getConfig().introspectClassAnnotations(referencedType);
+            deser = findDeserializerFromAnnotation(ctxt, refdDesc.getClassInfo());
+            return new AtomicReferenceDeserializer(referencedType, valueTypeDeser, deser);
         }
         return findOptionalStdDeserializer(ctxt, type, beanDesc);
     }
@@ -363,12 +366,14 @@ public class BeanDeserializerFactory
         }
         Class<?> implClass = objectIdInfo.getGeneratorType();
         JavaType idType;
-    	SettableBeanProperty idProp;
+        SettableBeanProperty idProp;
         ObjectIdGenerator<?> gen;
 
+        ObjectIdResolver resolver = ctxt.objectIdResolverInstance(beanDesc.getClassInfo(), objectIdInfo);
+
         // Just one special case: Property-based generator is trickier
         if (implClass == ObjectIdGenerators.PropertyGenerator.class) { // most special one, needs extra work
-            String propName = objectIdInfo.getPropertyName();
+            PropertyName propName = objectIdInfo.getPropertyName();
             idProp = builder.findProperty(propName);
             if (idProp == null) {
                 throw new IllegalArgumentException("Invalid Object Id definition for "
@@ -385,7 +390,7 @@ public class BeanDeserializerFactory
         // also: unlike with value deserializers, let's just resolve one we need here
         JsonDeserializer<?> deser = ctxt.findRootValueDeserializer(idType);
         builder.setObjectIdReader(ObjectIdReader.construct(idType,
-                objectIdInfo.getPropertyName(), gen, deser, idProp));
+                objectIdInfo.getPropertyName(), gen, deser, idProp, resolver));
     }
     
     @SuppressWarnings("unchecked")
@@ -532,28 +537,10 @@ public class BeanDeserializerFactory
         // At which point we still have all kinds of properties; not all with mutators:
         for (BeanPropertyDefinition propDef : propDefs) {
             SettableBeanProperty prop = null;
-            if (propDef.hasConstructorParameter()) {
-                /* [JACKSON-700] If property is passed via constructor parameter, we must
-                 *   handle things in special way. Not sure what is the most optimal way...
-                 *   for now, let's just call a (new) method in builder, which does nothing.
-                 */
-                // but let's call a method just to allow custom builders to be aware...
-                final String name = propDef.getName();
-                if (creatorProps != null) {
-                    for (SettableBeanProperty cp : creatorProps) {
-                        if (name.equals(cp.getName())) {
-                            prop = cp;
-                            break;
-                        }
-                    }
-                }
-                if (prop == null) {
-                    throw ctxt.mappingException("Could not find creator property with name '"
-                    		+name+"' (in class "+beanDesc.getBeanClass().getName()+")");
-                }
-                builder.addCreatorProperty(prop);
-                continue;
-            }
+            /* 18-Oct-2013, tatu: Although constructor parameters have highest precedence,
+             *   we need to do linkage (as per [Issue#318]), and so need to start with
+             *   other types, and only then create constructor parameter, if any.
+             */
             if (propDef.hasSetter()) {
                 Type propertyType = propDef.getSetter().getGenericParameterType(0);
                 prop = constructSettableProperty(ctxt, beanDesc, propDef, propertyType);
@@ -572,6 +559,34 @@ public class BeanDeserializerFactory
                     prop = constructSetterlessProperty(ctxt, beanDesc, propDef);
                 }
             }
+            if (propDef.hasConstructorParameter()) {
+                /* [JACKSON-700] If property is passed via constructor parameter, we must
+                 *   handle things in special way. Not sure what is the most optimal way...
+                 *   for now, let's just call a (new) method in builder, which does nothing.
+                 */
+                // but let's call a method just to allow custom builders to be aware...
+                final String name = propDef.getName();
+                CreatorProperty cprop = null;
+                if (creatorProps != null) {
+                    for (SettableBeanProperty cp : creatorProps) {
+                        if (name.equals(cp.getName())) {
+                            cprop = (CreatorProperty) cp;
+                            break;
+                        }
+                    }
+                }
+                if (cprop == null) {
+                    throw ctxt.mappingException("Could not find creator property with name '"
+                              +name+"' (in class "+beanDesc.getBeanClass().getName()+")");
+                }
+                if (prop != null) {
+                    cprop = cprop.withFallbackSetter(prop);
+                }
+                prop = cprop;
+                builder.addCreatorProperty(cprop);
+                continue;
+            }
+            
             if (prop != null) {
                 Class<?>[] views = propDef.findViews();
                 if (views == null) {
@@ -673,7 +688,8 @@ public class BeanDeserializerFactory
                 if (fixAccess) {
                     m.fixAccess(); // to ensure we can call it
                 }
-                builder.addInjectable(m.getName(), beanDesc.resolveType(m.getGenericType()),
+                builder.addInjectable(new PropertyName(m.getName()),
+                        beanDesc.resolveType(m.getGenericType()),
                         beanDesc.getClassAnnotations(), m, entry.getKey());
             }
         }
@@ -693,8 +709,9 @@ public class BeanDeserializerFactory
         }
         // we know it's a 2-arg method, second arg is the value
         JavaType type = beanDesc.bindingsForBeanType().resolveType(setter.getGenericParameterType(1));
-        BeanProperty.Std property = new BeanProperty.Std(setter.getName(), type, null,
-                beanDesc.getClassAnnotations(), setter, false);
+        BeanProperty.Std property = new BeanProperty.Std(new PropertyName(setter.getName()),
+                type, null, beanDesc.getClassAnnotations(), setter,
+                PropertyMetadata.STD_OPTIONAL);
         type = resolveType(ctxt, beanDesc, type, setter);
 
         /* AnySetter can be annotated with @JsonClass (etc) just like a
@@ -703,14 +720,16 @@ public class BeanDeserializerFactory
          * be done at a later point.
          */
         JsonDeserializer<Object> deser = findDeserializerFromAnnotation(ctxt, setter);
-        if (deser != null) {
-            return new SettableAnyProperty(property, setter, type, deser);
-        }
         /* Otherwise, method may specify more specific (sub-)class for
          * value (no need to check if explicit deser was specified):
          */
         type = modifyTypeByAnnotation(ctxt, setter, type);
-        return new SettableAnyProperty(property, setter, type, null);
+        if (deser == null) {
+            deser = type.getValueHandler();
+        }
+        TypeDeserializer typeDeser = type.getTypeHandler();
+        return new SettableAnyProperty(property, setter, type,
+                deser, typeDeser);
     }
 
     /**
@@ -726,15 +745,16 @@ public class BeanDeserializerFactory
         throws JsonMappingException
     {
         // need to ensure method is callable (for non-public)
-        AnnotatedMember mutator = propDef.getMutator();
+        AnnotatedMember mutator = propDef.getNonConstructorMutator();
         if (ctxt.canOverrideAccessModifiers()) {
             mutator.fixAccess();
         }
         // note: this works since we know there's exactly one argument for methods
         JavaType t0 = beanDesc.resolveType(jdkType);
 
-        BeanProperty.Std property = new BeanProperty.Std(propDef.getName(), t0, propDef.getWrapperName(),
-                beanDesc.getClassAnnotations(), mutator, propDef.isRequired());
+        BeanProperty.Std property = new BeanProperty.Std(propDef.getFullName(),
+                t0, propDef.getWrapperName(),
+                beanDesc.getClassAnnotations(), mutator, propDef.getMetadata());
         JavaType type = resolveType(ctxt, beanDesc, t0, mutator);
         // did type change?
         if (type != t0) {
@@ -763,6 +783,10 @@ public class BeanDeserializerFactory
         if (ref != null && ref.isManagedReference()) {
             prop.setManagedReferenceName(ref.getName());
         }
+        ObjectIdInfo objectIdInfo = propDef.findObjectIdInfo();
+        if(objectIdInfo != null){
+            prop.setObjectIdInfo(objectIdInfo);
+        }
         return prop;
     }
 
@@ -789,6 +813,8 @@ public class BeanDeserializerFactory
          */
         JsonDeserializer<Object> propDeser = findDeserializerFromAnnotation(ctxt, getter);
         type = modifyTypeByAnnotation(ctxt, getter, type);
+        // As per [Issue#501], need full resolution:
+        type = resolveType(ctxt, beanDesc, type, getter);
         TypeDeserializer typeDeser = type.getTypeHandler();
         SettableBeanProperty prop = new SetterlessProperty(propDef, type, typeDeser,
                 beanDesc.getClassAnnotations(), getter);
