@@ -163,7 +163,7 @@ public class BuilderBasedDeserializer
         case END_OBJECT: // added to resolve [JACKSON-319], possible related issues
             return finishBuild(ctxt, deserializeFromObject(jp, ctxt));
         default:
-            throw ctxt.mappingException(getBeanClass());
+            throw ctxt.mappingException(handledType());
         }
     }
 
@@ -227,16 +227,7 @@ public class BuilderBasedDeserializer
                 }
                 continue;
             }
-            if (_ignorableProps != null && _ignorableProps.contains(propName)) {
-                jp.skipChildren();
-            } else if (_anySetter != null) {
-                // should we try to get return value of any setter too?
-                _anySetter.deserializeAndSet(jp, ctxt, builder, propName);
-                continue;
-            } else {
-                // Unknown: let's call handler method
-                handleUnknownProperty(jp, ctxt, builder, propName);
-            }
+            handleUnknownVanilla(jp, ctxt, handledType(), propName);
         }
         return builder;
     }
@@ -308,22 +299,7 @@ public class BuilderBasedDeserializer
                 }
                 continue;
             }
-            /* As per [JACKSON-313], things marked as ignorable should not be
-             * passed to any setter
-             */
-            if (_ignorableProps != null && _ignorableProps.contains(propName)) {
-                jp.skipChildren();
-            } else if (_anySetter != null) {
-                try {
-                    _anySetter.deserializeAndSet(jp, ctxt, bean, propName);
-                } catch (Exception e) {
-                    wrapAndThrow(e, bean, propName, ctxt);
-                }
-                continue;
-            } else {
-                // Unknown: let's call handler method
-                handleUnknownProperty(jp, ctxt, bean, propName);         
-            }
+            handleUnknownVanilla(jp, ctxt, bean, propName);
         }
         return bean;
     }
@@ -337,6 +313,7 @@ public class BuilderBasedDeserializer
      * as well.
      */
     @Override
+    @SuppressWarnings("resource")
     protected final Object _deserializeUsingPropertyBased(final JsonParser jp,
             final DeserializationContext ctxt)
         throws IOException, JsonProcessingException
@@ -390,7 +367,7 @@ public class BuilderBasedDeserializer
             // As per [JACKSON-313], things marked as ignorable should not be
             // passed to any setter
             if (_ignorableProps != null && _ignorableProps.contains(propName)) {
-                jp.skipChildren();
+                handleIgnoredProperty(jp, ctxt, handledType(), propName);
                 continue;
             }
             // "any property"?
@@ -400,7 +377,7 @@ public class BuilderBasedDeserializer
             }
             // Ok then, let's collect the whole field; name and value
             if (unknown == null) {
-                unknown = new TokenBuffer(jp.getCodec());
+                unknown = new TokenBuffer(jp);
             }
             unknown.writeFieldName(propName);
             unknown.copyCurrentStructure(jp);
@@ -453,18 +430,7 @@ public class BuilderBasedDeserializer
                 }
                 continue;
             }
-            /* As per [JACKSON-313], things marked as ignorable should not be
-             * passed to any setter
-             */
-            if (_ignorableProps != null && _ignorableProps.contains(propName)) {
-                jp.skipChildren();
-            } else if (_anySetter != null) {
-                _anySetter.deserializeAndSet(jp, ctxt, bean, propName);
-                continue;
-            } else {
-                // Unknown: let's call handler method
-                handleUnknownProperty(jp, ctxt, bean, propName);
-            }
+            handleUnknownVanilla(jp, ctxt, bean, propName);
         }
         return bean;
     }
@@ -479,6 +445,7 @@ public class BuilderBasedDeserializer
      * Method called when there are declared "unwrapped" properties
      * which need special handling
      */
+    @SuppressWarnings("resource")
     protected Object deserializeWithUnwrapped(JsonParser jp, DeserializationContext ctxt)
         throws IOException, JsonProcessingException
     {
@@ -488,7 +455,7 @@ public class BuilderBasedDeserializer
         if (_propertyBasedCreator != null) {
             return deserializeUsingPropertyBasedWithUnwrapped(jp, ctxt);
         }
-        TokenBuffer tokens = new TokenBuffer(jp.getCodec());
+        TokenBuffer tokens = new TokenBuffer(jp);
         tokens.writeStartObject();
         Object bean = _valueInstantiator.createUsingDefault(ctxt);
 
@@ -516,7 +483,7 @@ public class BuilderBasedDeserializer
             }
             // ignorable things should be ignored
             if (_ignorableProps != null && _ignorableProps.contains(propName)) {
-                jp.skipChildren();
+                handleIgnoredProperty(jp, ctxt, bean, propName);
                 continue;
             }
             // but... others should be passed to unwrapped property deserializers
@@ -537,6 +504,7 @@ public class BuilderBasedDeserializer
         return bean;
     }    
 
+    @SuppressWarnings("resource")
     protected Object deserializeWithUnwrapped(JsonParser jp,
     		DeserializationContext ctxt, Object bean)
         throws IOException, JsonProcessingException
@@ -545,7 +513,7 @@ public class BuilderBasedDeserializer
         if (t == JsonToken.START_OBJECT) {
             t = jp.nextToken();
         }
-        TokenBuffer tokens = new TokenBuffer(jp.getCodec());
+        TokenBuffer tokens = new TokenBuffer(jp);
         tokens.writeStartObject();
         final Class<?> activeView = _needViewProcesing ? ctxt.getActiveView() : null;
         for (; t == JsonToken.FIELD_NAME; t = jp.nextToken()) {
@@ -565,7 +533,7 @@ public class BuilderBasedDeserializer
                 continue;
             }
             if (_ignorableProps != null && _ignorableProps.contains(propName)) {
-                jp.skipChildren();
+                handleIgnoredProperty(jp, ctxt, bean, propName);
                 continue;
             }
             // but... others should be passed to unwrapped property deserializers
@@ -589,7 +557,7 @@ public class BuilderBasedDeserializer
         final PropertyBasedCreator creator = _propertyBasedCreator;
         PropertyValueBuffer buffer = creator.startBuilding(jp, ctxt, _objectIdReader);
 
-        TokenBuffer tokens = new TokenBuffer(jp.getCodec());
+        TokenBuffer tokens = new TokenBuffer(jp);
         tokens.writeStartObject();
 
         JsonToken t = jp.getCurrentToken();
@@ -636,11 +604,8 @@ public class BuilderBasedDeserializer
                 buffer.bufferProperty(prop, prop.deserialize(jp, ctxt));
                 continue;
             }
-            /* As per [JACKSON-313], things marked as ignorable should not be
-             * passed to any setter
-             */
             if (_ignorableProps != null && _ignorableProps.contains(propName)) {
-                jp.skipChildren();
+                handleIgnoredProperty(jp, ctxt, handledType(), propName);
                 continue;
             }
             tokens.writeFieldName(propName);
@@ -703,7 +668,7 @@ public class BuilderBasedDeserializer
             }
             // ignorable things should be ignored
             if (_ignorableProps != null && _ignorableProps.contains(propName)) {
-                jp.skipChildren();
+                handleIgnoredProperty(jp, ctxt, bean, propName);
                 continue;
             }
             // but others are likely to be part of external type id thingy...
