@@ -4,30 +4,29 @@ import java.io.IOException;
 import java.lang.annotation.Annotation;
 
 import com.fasterxml.jackson.core.*;
-import com.fasterxml.jackson.core.util.InternCache;
 import com.fasterxml.jackson.databind.*;
+import com.fasterxml.jackson.databind.deser.impl.FailingDeserializer;
 import com.fasterxml.jackson.databind.deser.impl.NullProvider;
 import com.fasterxml.jackson.databind.introspect.AnnotatedMember;
 import com.fasterxml.jackson.databind.introspect.BeanPropertyDefinition;
+import com.fasterxml.jackson.databind.introspect.ObjectIdInfo;
 import com.fasterxml.jackson.databind.jsonFormatVisitors.JsonObjectFormatVisitor;
 import com.fasterxml.jackson.databind.jsontype.TypeDeserializer;
-import com.fasterxml.jackson.databind.jsontype.impl.FailingDeserializer;
 import com.fasterxml.jackson.databind.util.Annotations;
 import com.fasterxml.jackson.databind.util.ViewMatcher;
 
 /**
- * Base class for deserilizable properties of a bean: contains
+ * Base class for deserializable properties of a bean: contains
  * both type and name definitions, and reflection-based set functionality.
  * Concrete sub-classes implement details, so that field- and
  * setter-backed properties, as well as a few more esoteric variations,
  * can be handled.
  */
+@SuppressWarnings("serial")
 public abstract class SettableBeanProperty
     implements BeanProperty,
         java.io.Serializable
 {
-    private static final long serialVersionUID = -1026580169193933453L;
-
     /**
      * To avoid nasty NPEs, let's use a placeholder for _valueDeserializer,
      * if real deserializer is not (yet) available.
@@ -41,7 +40,7 @@ public abstract class SettableBeanProperty
      * Logical name of the property (often but not always derived
      * from the setter method name)
      */
-    protected final String _propName;
+    protected final PropertyName _propName;
 
     /**
      * Base type for property; may be a supertype of actual value.
@@ -62,8 +61,10 @@ public abstract class SettableBeanProperty
     
     /**
      * Deserializer used for handling property value.
+     *<p>
+     * NOTE: has been immutable since 2.3
      */
-    protected JsonDeserializer<Object> _valueDeserializer;
+    protected final JsonDeserializer<Object> _valueDeserializer;
 
     /**
      * If value will contain type information (to support
@@ -80,14 +81,13 @@ public abstract class SettableBeanProperty
     protected final NullProvider _nullProvider;
 
     /**
-     * Whether value of this property has been marked as required.
-     * Retained since it will be needed when traversing type hierarchy
-     * for producing schemas (and other similar tasks); currently not
-     * used for serialization.
+     * Additional optional property metadata, such as whether
+     * property is required, and whether there is additional
+     * human-readable description
      * 
-     * @since 2.2
+     * @since 2.3
      */
-    protected final boolean _isRequired;
+    protected final PropertyMetadata _metadata;
 
     /*
     /**********************************************************
@@ -106,6 +106,13 @@ public abstract class SettableBeanProperty
      */
     protected String _managedReferenceName;
 
+    /**
+     * This is the information for object identity associated with the property.
+     * <p>
+     * TODO: should try to make immutable.
+     */
+    protected ObjectIdInfo _objectIdInfo;
+
     /**
      * Helper object used for checking whether this property is to
      * be included in the active view, if property is view-specific;
@@ -124,7 +131,7 @@ public abstract class SettableBeanProperty
      * TODO: should try to make immutable if at all possible
      */
     protected int _propertyIndex = -1;
-    
+
     /*
     /**********************************************************
     /* Life-cycle (construct & configure)
@@ -134,36 +141,44 @@ public abstract class SettableBeanProperty
     protected SettableBeanProperty(BeanPropertyDefinition propDef,
             JavaType type, TypeDeserializer typeDeser, Annotations contextAnnotations)
     {
-        this(propDef.getName(), type, propDef.getWrapperName(), typeDeser, contextAnnotations,
-                propDef.isRequired());
+        this(propDef.getFullName(), type, propDef.getWrapperName(), typeDeser,
+                contextAnnotations, propDef.getMetadata());
     }
 
     @Deprecated // since 2.2
     protected SettableBeanProperty(String propName, JavaType type, PropertyName wrapper,
             TypeDeserializer typeDeser, Annotations contextAnnotations)
     {
-        this(propName, type, wrapper, typeDeser, contextAnnotations, false);
+        this(new PropertyName(propName), type, wrapper, typeDeser, contextAnnotations,
+                PropertyMetadata.STD_OPTIONAL);
     }
-    
+
+    @Deprecated // since 2.3
     protected SettableBeanProperty(String propName, JavaType type, PropertyName wrapper,
             TypeDeserializer typeDeser, Annotations contextAnnotations,
             boolean isRequired)
     {
-        /* 09-Jan-2009, tatu: Intern()ing makes sense since Jackson parsed
-         *   field names are (usually) interned too, hence lookups will be faster.
-         */
+        this(new PropertyName(propName), type, wrapper, typeDeser, contextAnnotations,
+                PropertyMetadata.construct(isRequired, null, null));
+    }
+    
+    protected SettableBeanProperty(PropertyName propName, JavaType type, PropertyName wrapper,
+            TypeDeserializer typeDeser, Annotations contextAnnotations,
+            PropertyMetadata metadata)
+    {
+        // 09-Jan-2009, tatu: Intern()ing makes sense since Jackson parsed
+        //  field names are (usually) interned too, hence lookups will be faster.
         // 23-Oct-2009, tatu: should this be disabled wrt [JACKSON-180]?
-        /*   Probably need not, given that namespace of field/method names
-         *   is not unbounded, unlike potential JSON names.
-         */
-        if (propName == null || propName.length() == 0) {
-            _propName = "";
+        //   Probably need not, given that namespace of field/method names
+        //   is not unbounded, unlike potential JSON names.
+        if (propName == null) {
+            _propName = PropertyName.NO_NAME;
         } else {
-            _propName = InternCache.instance.intern(propName);
+            _propName = propName.internSimpleName();
         }
         _type = type;
         _wrapperName = wrapper;
-        _isRequired = isRequired;
+        _metadata = metadata;
         _contextAnnotations = contextAnnotations;
         _viewMatcher = null;
         _nullProvider = null;
@@ -176,6 +191,30 @@ public abstract class SettableBeanProperty
         _valueDeserializer = MISSING_VALUE_DESERIALIZER;
     }
 
+    /**
+     * Constructor only used by {@link com.fasterxml.jackson.databind.deser.impl.ObjectIdValueProperty}.
+     * 
+     * @since 2.3
+     */
+    protected SettableBeanProperty(PropertyName propName, JavaType type, 
+            PropertyMetadata metadata, JsonDeserializer<Object> valueDeser)
+    {
+        // as with above ctor, intern()ing probably fine
+        if (propName == null) {
+            _propName = PropertyName.NO_NAME;
+        } else {
+            _propName = propName.internSimpleName();
+        }
+        _type = type;
+        _wrapperName = null;
+        _metadata = metadata;
+        _contextAnnotations = null;
+        _viewMatcher = null;
+        _nullProvider = null;
+        _valueTypeDeserializer = null;
+        _valueDeserializer = valueDeser;
+    }
+    
     /**
      * Basic copy-constructor for sub-classes to use.
      */
@@ -184,7 +223,7 @@ public abstract class SettableBeanProperty
         _propName = src._propName;
         _type = src._type;
         _wrapperName = src._wrapperName;
-        _isRequired = src._isRequired;
+        _metadata = src._metadata;
         _contextAnnotations = src._contextAnnotations;
         _valueDeserializer = src._valueDeserializer;
         _valueTypeDeserializer = src._valueTypeDeserializer;
@@ -203,7 +242,7 @@ public abstract class SettableBeanProperty
         _propName = src._propName;
         _type = src._type;
         _wrapperName = src._wrapperName;
-        _isRequired = src._isRequired;
+        _metadata = src._metadata;
         _contextAnnotations = src._contextAnnotations;
         _valueTypeDeserializer = src._valueTypeDeserializer;
         _managedReferenceName = src._managedReferenceName;
@@ -220,15 +259,20 @@ public abstract class SettableBeanProperty
         _viewMatcher = src._viewMatcher;
     }
 
+    @Deprecated // since 2.3
+    protected SettableBeanProperty(SettableBeanProperty src, String newName) {
+        this(src, new PropertyName(newName));
+    }
+    
     /**
      * Copy-with-deserializer-change constructor for sub-classes to use.
      */
-    protected SettableBeanProperty(SettableBeanProperty src, String newName)
+    protected SettableBeanProperty(SettableBeanProperty src, PropertyName newName)
     {
         _propName = newName;
         _type = src._type;
         _wrapperName = src._wrapperName;
-        _isRequired = src._isRequired;
+        _metadata = src._metadata;
         _contextAnnotations = src._contextAnnotations;
         _valueDeserializer = src._valueDeserializer;
         _valueTypeDeserializer = src._valueTypeDeserializer;
@@ -260,12 +304,30 @@ public abstract class SettableBeanProperty
      * @return Newly constructed instance, if property name differs from the
      *   one used for this instance; or 'this' if not.
      */
-    public abstract SettableBeanProperty withName(String newName);
+    public abstract SettableBeanProperty withName(PropertyName newName);
+
+    /**
+     * @since 2.3
+     */
+    public SettableBeanProperty withSimpleName(String simpleName) {
+        PropertyName n = (_propName == null)
+                ? new PropertyName(simpleName) : _propName.withSimpleName(simpleName);
+        return (n == _propName) ? this : withName(n);
+    }
     
+    @Deprecated // since 2.3 -- use 'withSimpleName' instead if need be
+    public SettableBeanProperty withName(String simpleName) {
+        return withName(new PropertyName(simpleName));
+    }
+
     public void setManagedReferenceName(String n) {
         _managedReferenceName = n;
     }
-    
+
+    public void setObjectIdInfo(ObjectIdInfo objectIdInfo) {
+        _objectIdInfo = objectIdInfo;
+    }
+
     public void setViews(Class<?>[] views) {
         if (views == null) {
             _viewMatcher = null;
@@ -291,10 +353,20 @@ public abstract class SettableBeanProperty
      */
     
     @Override
-    public final String getName() { return _propName; }
+    public final String getName() {
+        return _propName.getSimpleName();
+    }
 
     @Override
-    public boolean isRequired() { return _isRequired; }
+    public PropertyName getFullName() {
+        return _propName;
+    }
+    
+    @Override
+    public boolean isRequired() { return _metadata.isRequired(); }
+
+    @Override
+    public PropertyMetadata getMetadata() { return _metadata; }
     
     @Override
     public JavaType getType() { return _type; }
@@ -338,6 +410,8 @@ public abstract class SettableBeanProperty
 
     public String getManagedReferenceName() { return _managedReferenceName; }
 
+    public ObjectIdInfo getObjectIdInfo() { return _objectIdInfo; }
+
     public boolean hasValueDeserializer() {
         return (_valueDeserializer != null) && (_valueDeserializer != MISSING_VALUE_DESERIALIZER);
     }
