@@ -10,7 +10,7 @@ import com.fasterxml.jackson.databind.annotation.JacksonStdImpl;
 import com.fasterxml.jackson.databind.deser.*;
 import com.fasterxml.jackson.databind.deser.impl.PropertyBasedCreator;
 import com.fasterxml.jackson.databind.deser.impl.PropertyValueBuffer;
-import com.fasterxml.jackson.databind.deser.std.ContainerDeserializerBase;
+import com.fasterxml.jackson.databind.deser.impl.ReadableObjectId.Referring;
 import com.fasterxml.jackson.databind.jsontype.TypeDeserializer;
 import com.fasterxml.jackson.databind.util.ArrayBuilders;
 
@@ -95,7 +95,7 @@ public class MapDeserializer
             KeyDeserializer keyDeser, JsonDeserializer<Object> valueDeser,
             TypeDeserializer valueTypeDeser)
     {
-        super(Map.class);
+        super(mapType);
         _mapType = mapType;
         _keyDeserializer = keyDeser;
         _valueDeserializer = valueDeser;
@@ -113,7 +113,7 @@ public class MapDeserializer
      */
     protected MapDeserializer(MapDeserializer src)
     {
-        super(src._valueClass);
+        super(src._mapType);
         _mapType = src._mapType;
         _keyDeserializer = src._keyDeserializer;
         _valueDeserializer = src._valueDeserializer;
@@ -133,7 +133,7 @@ public class MapDeserializer
             TypeDeserializer valueTypeDeser,
             HashSet<String> ignorable)
     {
-        super(src._valueClass);
+        super(src._mapType);
         _mapType = src._mapType;
         _keyDeserializer = keyDeser;
         _valueDeserializer = valueDeser;
@@ -157,8 +157,8 @@ public class MapDeserializer
             HashSet<String> ignorable)
     {
         
-        if ((_keyDeserializer == keyDeser) && (_valueDeserializer == valueDeser) && (_valueTypeDeserializer == valueTypeDeser)
-                && (_ignorableProperties == ignorable)) {
+        if ((_keyDeserializer == keyDeser) && (_valueDeserializer == valueDeser)
+                && (_valueTypeDeserializer == valueTypeDeser) && (_ignorableProperties == ignorable)) {
             return this;
         }
         return new MapDeserializer(this,
@@ -183,8 +183,7 @@ public class MapDeserializer
                 && isDefaultKeyDeserializer(keyDeser));
     }
     
-    public void setIgnorableProperties(String[] ignorable)
-    {
+    public void setIgnorableProperties(String[] ignorable) {
         _ignorableProperties = (ignorable == null || ignorable.length == 0) ?
             null : ArrayBuilders.arrayToSet(ignorable);
     }
@@ -241,9 +240,7 @@ public class MapDeserializer
         if (vd == null) {
             vd = ctxt.findContextualValueDeserializer(_mapType.getContentType(), property);
         } else { // if directly assigned, probably not yet contextual, so:
-            if (vd instanceof ContextualDeserializer) {
-                vd = ((ContextualDeserializer) vd).createContextual(ctxt, property);
-            }
+            vd = ctxt.handleSecondaryContextualization(vd, property);
         }
         TypeDeserializer vtd = _valueTypeDeserializer;
         if (vtd != null) {
@@ -373,6 +370,12 @@ public class MapDeserializer
         final KeyDeserializer keyDes = _keyDeserializer;
         final JsonDeserializer<Object> valueDes = _valueDeserializer;
         final TypeDeserializer typeDeser = _valueTypeDeserializer;
+
+        MapReferringAccumulator referringAccumulator = null;
+        boolean useObjectId = valueDes.getObjectIdReader() != null;
+        if (useObjectId) {
+            referringAccumulator = new MapReferringAccumulator(_mapType.getContentType().getRawClass(), result);
+        }
         for (; t == JsonToken.FIELD_NAME; t = jp.nextToken()) {
             // Must point to field name
             String fieldName = jp.getCurrentName();
@@ -383,20 +386,28 @@ public class MapDeserializer
                 jp.skipChildren();
                 continue;
             }
-            // Note: must handle null explicitly here; value deserializers won't
-            Object value;            
-            if (t == JsonToken.VALUE_NULL) {
-                value = null;
-            } else if (typeDeser == null) {
-                value = valueDes.deserialize(jp, ctxt);
-            } else {
-                value = valueDes.deserializeWithType(jp, ctxt, typeDeser);
+            try{
+                // Note: must handle null explicitly here; value deserializers won't
+                Object value;
+                if (t == JsonToken.VALUE_NULL) {
+                    value = valueDes.getNullValue();
+                } else if (typeDeser == null) {
+                    value = valueDes.deserialize(jp, ctxt);
+                } else {
+                    value = valueDes.deserializeWithType(jp, ctxt, typeDeser);
+                }
+                /* !!! 23-Dec-2008, tatu: should there be an option to verify
+                 *   that there are no duplicate field names? (and/or what
+                 *   to do, keep-first or keep-last)
+                 */
+                if (useObjectId) {
+                    referringAccumulator.put(key, value);
+                } else {
+                    result.put(key, value);
+                }
+            } catch(UnresolvedForwardReference reference) {
+                handleUnresolvedReference(jp, referringAccumulator, key, reference);
             }
-            /* !!! 23-Dec-2008, tatu: should there be an option to verify
-             *   that there are no duplicate field names? (and/or what
-             *   to do, keep-first or keep-last)
-             */
-            result.put(key, value);
         }
     }
 
@@ -415,6 +426,11 @@ public class MapDeserializer
         }
         final JsonDeserializer<Object> valueDes = _valueDeserializer;
         final TypeDeserializer typeDeser = _valueTypeDeserializer;
+        MapReferringAccumulator referringAccumulator = null;
+        boolean useObjectId = valueDes.getObjectIdReader() != null;
+        if (useObjectId) {
+            referringAccumulator = new MapReferringAccumulator(_mapType.getContentType().getRawClass(), result);
+        }
         for (; t == JsonToken.FIELD_NAME; t = jp.nextToken()) {
             // Must point to field name
             String fieldName = jp.getCurrentName();
@@ -424,16 +440,24 @@ public class MapDeserializer
                 jp.skipChildren();
                 continue;
             }
-            // Note: must handle null explicitly here; value deserializers won't
-            Object value;            
-            if (t == JsonToken.VALUE_NULL) {
-                value = null;
-            } else if (typeDeser == null) {
-                value = valueDes.deserialize(jp, ctxt);
-            } else {
-                value = valueDes.deserializeWithType(jp, ctxt, typeDeser);
+            try {
+                // Note: must handle null explicitly here; value deserializers won't
+                Object value;
+                if (t == JsonToken.VALUE_NULL) {
+                    value = valueDes.getNullValue();
+                } else if (typeDeser == null) {
+                    value = valueDes.deserialize(jp, ctxt);
+                } else {
+                    value = valueDes.deserializeWithType(jp, ctxt, typeDeser);
+                }
+                if (useObjectId) {
+                    referringAccumulator.put(fieldName, value);
+                } else {
+                    result.put(fieldName, value);
+                }
+            } catch (UnresolvedForwardReference reference) {
+                handleUnresolvedReference(jp, referringAccumulator, fieldName, reference);
             }
-            result.put(fieldName, value);
         }
     }
     
@@ -482,7 +506,7 @@ public class MapDeserializer
             Object key = _keyDeserializer.deserializeKey(fieldName, ctxt);
             Object value;            
             if (t == JsonToken.VALUE_NULL) {
-                value = null;
+                value = valueDes.getNullValue();
             } else if (typeDeser == null) {
                 value = valueDes.deserialize(jp, ctxt);
             } else {
@@ -500,7 +524,7 @@ public class MapDeserializer
         }
     }
 
-    // note: copied form BeanDeserializer; should try to share somehow...
+    // note: copied from BeanDeserializer; should try to share somehow...
     protected void wrapAndThrow(Throwable t, Object ref)
         throws IOException
     {
@@ -518,4 +542,93 @@ public class MapDeserializer
         }
         throw JsonMappingException.wrapWithPath(t, ref, null);
     }
+
+    private void handleUnresolvedReference(JsonParser jp, MapReferringAccumulator accumulator, Object key,
+            UnresolvedForwardReference reference)
+        throws JsonMappingException
+    {
+        if (accumulator == null) {
+            throw JsonMappingException.from(jp, "Unresolved forward reference but no identity info.", reference);
+        }
+        Referring referring = accumulator.handleUnresolvedReference(reference, key);
+        reference.getRoid().appendReferring(referring);
+    }
+
+    private final static class MapReferringAccumulator  {
+        private final Class<?> _valueType;
+        private Map<Object,Object> _result;
+        /**
+         * A list of {@link MapReferring} to maintain ordering.
+         */
+        private List<MapReferring> _accumulator = new ArrayList<MapReferring>();
+
+        public MapReferringAccumulator(Class<?> valueType, Map<Object, Object> result) {
+            _valueType = valueType;
+            _result = result;
+        }
+
+        public void put(Object key, Object value)
+        {
+            if (_accumulator.isEmpty()) {
+                _result.put(key, value);
+            } else {
+                MapReferring ref = _accumulator.get(_accumulator.size() - 1);
+                ref.next.put(key, value);
+            }
+        }
+
+        public Referring handleUnresolvedReference(UnresolvedForwardReference reference, Object key)
+        {
+            MapReferring id = new MapReferring(this, reference, _valueType, key);
+            _accumulator.add(id);
+            return id;
+        }
+
+        public void resolveForwardReference(Object id, Object value) throws IOException
+        {
+            Iterator<MapReferring> iterator = _accumulator.iterator();
+            // Resolve ordering after resolution of an id. This means either:
+            // 1- adding to the result map in case of the first unresolved id.
+            // 2- merge the content of the resolved id with its previous unresolved id.
+            Map<Object,Object> previous = _result;
+            while (iterator.hasNext()) {
+                MapReferring ref = iterator.next();
+                if (ref.hasId(id)) {
+                    iterator.remove();
+                    previous.put(ref.key, value);
+                    previous.putAll(ref.next);
+                    return;
+                }
+                previous = ref.next;
+            }
+
+            throw new IllegalArgumentException("Trying to resolve a forward reference with id [" + id
+                    + "] that wasn't previously seen as unresolved.");
+        }
+    }
+
+    /**
+     * Helper class to maintain processing order of value. The resolved
+     * object associated with {@link #_id} comes before the values in
+     * {@link _next}.
+     */
+    private final static class MapReferring extends Referring {
+        private final MapReferringAccumulator _parent;
+
+        public final Map<Object, Object> next = new LinkedHashMap<Object, Object>();
+        public final Object key;
+        
+        private MapReferring(MapReferringAccumulator parent, UnresolvedForwardReference ref,
+                Class<?> valueType, Object key)
+        {
+            super(ref, valueType);
+            _parent = parent;
+            this.key = key;
+        }
+
+        @Override
+        public void handleResolvedForwardReference(Object id, Object value) throws IOException {
+            _parent.resolveForwardReference(id, value);
+        }
+    }
 }
