@@ -8,7 +8,6 @@ import java.util.HashSet;
 import com.fasterxml.jackson.core.JsonParser;
 import com.fasterxml.jackson.core.JsonProcessingException;
 import com.fasterxml.jackson.core.JsonToken;
-
 import com.fasterxml.jackson.databind.DeserializationContext;
 import com.fasterxml.jackson.databind.DeserializationFeature;
 import com.fasterxml.jackson.databind.JsonDeserializer;
@@ -44,44 +43,7 @@ public class NumberDeserializers
         }
     }
 
-    /**
-     * @deprecated Since 2.2 -- use {@link #find} instead.
-     */
-    @Deprecated
-    public static StdDeserializer<?>[] all()
-    {
-        return new StdDeserializer<?>[] {
-                // primitive-wrappers (simple):
-                new BooleanDeserializer(Boolean.class, null),
-                new ByteDeserializer(Byte.class, null),
-                new ShortDeserializer(Short.class, null),
-                new CharacterDeserializer(Character.class, null),
-                new IntegerDeserializer(Integer.class, null),
-                new LongDeserializer(Long.class, null),
-                new FloatDeserializer(Float.class, null),
-                new DoubleDeserializer(Double.class, null),
-
-                /* And actual primitives: difference is the way nulls are to be
-                 * handled...
-                 */
-                new BooleanDeserializer(Boolean.TYPE, Boolean.FALSE),
-                new ByteDeserializer(Byte.TYPE, Byte.valueOf((byte)(0))),
-                new ShortDeserializer(Short.TYPE, Short.valueOf((short)0)),
-                new CharacterDeserializer(Character.TYPE, Character.valueOf('\0')),
-                new IntegerDeserializer(Integer.TYPE, Integer.valueOf(0)),
-                new LongDeserializer(Long.TYPE, Long.valueOf(0L)),
-                new FloatDeserializer(Float.TYPE, Float.valueOf(0.0f)),
-                new DoubleDeserializer(Double.TYPE, Double.valueOf(0.0)),
-                
-                // and related
-                new NumberDeserializer(),
-                new BigDecimalDeserializer(),
-                new BigIntegerDeserializer()
-        };
-    }
-    
-    public static JsonDeserializer<?> find(Class<?> rawType, String clsName)
-    {
+    public static JsonDeserializer<?> find(Class<?> rawType, String clsName) {
         if (rawType.isPrimitive()) {
             if (rawType == Integer.TYPE) {
                 return IntegerDeserializer.primitiveInstance;
@@ -163,8 +125,7 @@ public class NumberDeserializers
 
         protected final T _nullValue;
         
-        protected PrimitiveOrWrapperDeserializer(Class<T> vc, T nvl)
-        {
+        protected PrimitiveOrWrapperDeserializer(Class<T> vc, T nvl) {
             super(vc);
             _nullValue = nvl;
         }
@@ -214,7 +175,7 @@ public class NumberDeserializers
     }
 
     @JacksonStdImpl
-    public final static class ByteDeserializer
+    public static class ByteDeserializer
         extends PrimitiveOrWrapperDeserializer<Byte>
     {
         private static final long serialVersionUID = 1L;
@@ -236,7 +197,7 @@ public class NumberDeserializers
     }
 
     @JacksonStdImpl
-    public final static class ShortDeserializer
+    public static class ShortDeserializer
         extends PrimitiveOrWrapperDeserializer<Short>
     {
         private static final long serialVersionUID = 1L;
@@ -258,7 +219,7 @@ public class NumberDeserializers
     }
 
     @JacksonStdImpl
-    public final static class CharacterDeserializer
+    public static class CharacterDeserializer
         extends PrimitiveOrWrapperDeserializer<Character>
     {
         private static final long serialVersionUID = 1L;
@@ -276,10 +237,9 @@ public class NumberDeserializers
             throws IOException, JsonProcessingException
         {
             JsonToken t = jp.getCurrentToken();
-            int value;
-
+            
             if (t == JsonToken.VALUE_NUMBER_INT) { // ok iff ascii value
-                value = jp.getIntValue();
+                int value = jp.getIntValue();
                 if (value >= 0 && value <= 0xFFFF) {
                     return Character.valueOf((char) value);
                 }
@@ -292,7 +252,17 @@ public class NumberDeserializers
                 // actually, empty should become null?
                 if (text.length() == 0) {
                     return (Character) getEmptyValue();
+                }               
+            } else if (t == JsonToken.START_ARRAY && ctxt.isEnabled(DeserializationFeature.UNWRAP_SINGLE_VALUE_ARRAYS)) {
+                //Issue#381
+                jp.nextToken();
+                final Character value = deserialize(jp, ctxt);
+                if (jp.nextToken() != JsonToken.END_ARRAY) {
+                    throw ctxt.wrongTokenException(jp, JsonToken.END_ARRAY, 
+                            "Attempted to unwrap single value array for single '" + _valueClass.getName() + "' value but there was more than a single value in the array"
+                            );
                 }
+                return value;
             }
             throw ctxt.mappingException(_valueClass, t);
         }
@@ -353,7 +323,7 @@ public class NumberDeserializers
     }
 
     @JacksonStdImpl
-    public final static class FloatDeserializer
+    public static class FloatDeserializer
         extends PrimitiveOrWrapperDeserializer<Float>
     {
         private static final long serialVersionUID = 1L;
@@ -378,7 +348,7 @@ public class NumberDeserializers
     }
 
     @JacksonStdImpl
-    public final static class DoubleDeserializer
+    public static class DoubleDeserializer
         extends PrimitiveOrWrapperDeserializer<Double>
     {
         private static final long serialVersionUID = 1L;
@@ -421,7 +391,7 @@ public class NumberDeserializers
      */
     @SuppressWarnings("serial")
     @JacksonStdImpl
-    public final static class NumberDeserializer
+    public static class NumberDeserializer
         extends StdScalarDeserializer<Number>
     {
         public final static NumberDeserializer instance = new NumberDeserializer();
@@ -453,15 +423,28 @@ public class NumberDeserializers
              */
             if (t == JsonToken.VALUE_STRING) { // let's do implicit re-parse
                 String text = jp.getText().trim();
+                if (text.length() == 0) {
+                    return getEmptyValue();
+                }
+                if (_hasTextualNull(text)) {
+                    return getNullValue();
+                }
+                if (_isPosInf(text)) {
+                    return Double.POSITIVE_INFINITY;
+                }
+                if (_isNegInf(text)) {
+                    return Double.NEGATIVE_INFINITY;
+                }
+                if (_isNaN(text)) {
+                    return Double.NaN;
+                }
                 try {
                     if (text.indexOf('.') >= 0) { // floating point
-                        // as per [JACKSON-72]:
                         if (ctxt.isEnabled(DeserializationFeature.USE_BIG_DECIMAL_FOR_FLOATS)) {
                             return new BigDecimal(text);
                         }
                         return new Double(text);
                     }
-                    // as per [JACKSON-100]:
                     if (ctxt.isEnabled(DeserializationFeature.USE_BIG_INTEGER_FOR_INTS)) {
                         return new BigInteger(text);
                     }
@@ -474,6 +457,17 @@ public class NumberDeserializers
                     throw ctxt.weirdStringException(text, _valueClass, "not a valid number");
                 }
             }
+            
+            if (t == JsonToken.START_ARRAY && ctxt.isEnabled(DeserializationFeature.UNWRAP_SINGLE_VALUE_ARRAYS)) {
+                jp.nextToken();
+                final Number value = deserialize(jp, ctxt);
+                if (jp.nextToken() != JsonToken.END_ARRAY) {
+                    throw ctxt.wrongTokenException(jp, JsonToken.END_ARRAY, 
+                            "Attempted to unwrap single value array for single '" + _valueClass.getName() + "' value but there was more than a single value in the array"
+                            );
+                }
+                return value;
+            }
             // Otherwise, no can do:
             throw ctxt.mappingException(_valueClass, t);
         }
@@ -540,10 +534,19 @@ public class NumberDeserializers
                  * Could do by calling BigDecimal.toBigIntegerExact()
                  */
                 return jp.getDecimalValue().toBigInteger();
+            } else if (t == JsonToken.START_ARRAY && ctxt.isEnabled(DeserializationFeature.UNWRAP_SINGLE_VALUE_ARRAYS)) {
+                jp.nextToken();
+                final BigInteger value = deserialize(jp, ctxt);
+                if (jp.nextToken() != JsonToken.END_ARRAY) {
+                    throw ctxt.wrongTokenException(jp, JsonToken.END_ARRAY,
+                        "Attempted to unwrap single value array for single 'BigInteger' value but there was more than a single value in the array"
+                    );
+                }
+                return value;
             } else if (t != JsonToken.VALUE_STRING) { // let's do implicit re-parse
                 // String is ok too, can easily convert; otherwise, no can do:
                 throw ctxt.mappingException(_valueClass, t);
-            }
+            }            
             text = jp.getText().trim();
             if (text.length() == 0) {
                 return null;
@@ -585,6 +588,17 @@ public class NumberDeserializers
                     throw ctxt.weirdStringException(text, _valueClass, "not a valid representation");
                 }
             }
+            
+            if (t == JsonToken.START_ARRAY && ctxt.isEnabled(DeserializationFeature.UNWRAP_SINGLE_VALUE_ARRAYS)) {
+                jp.nextToken();
+                final BigDecimal value = deserialize(jp, ctxt);
+                if (jp.nextToken() != JsonToken.END_ARRAY) {
+                    throw ctxt.wrongTokenException(jp, JsonToken.END_ARRAY,
+                        "Attempted to unwrap single value array for single 'BigDecimal' value but there was more than a single value in the array"
+                    );
+                }
+                return value;
+            }
             // Otherwise, no can do:
             throw ctxt.mappingException(_valueClass, t);
         }
