@@ -7,9 +7,7 @@ import java.util.*;
 import com.fasterxml.jackson.annotation.JsonFormat;
 import com.fasterxml.jackson.annotation.ObjectIdGenerator;
 import com.fasterxml.jackson.annotation.ObjectIdGenerators;
-
 import com.fasterxml.jackson.core.*;
-
 import com.fasterxml.jackson.databind.*;
 import com.fasterxml.jackson.databind.introspect.Annotated;
 import com.fasterxml.jackson.databind.introspect.AnnotatedMember;
@@ -40,6 +38,8 @@ public abstract class BeanSerializerBase
     implements ContextualSerializer, ResolvableSerializer,
         JsonFormatVisitable, SchemaAware
 {
+    protected final static PropertyName NAME_FOR_OBJECT_REF = new PropertyName("#object-ref");
+    
     final protected static BeanPropertyWriter[] NO_PROPS = new BeanPropertyWriter[0];
 
     /*
@@ -64,7 +64,7 @@ public abstract class BeanSerializerBase
      * annotated properties
      */
     final protected AnyGetterWriter _anyGetterWriter;
-    
+
     /**
      * Id of the bean property filter to use, if any; null if none.
      */
@@ -137,7 +137,17 @@ public abstract class BeanSerializerBase
         _serializationShape = src._serializationShape;
     }
 
-    protected BeanSerializerBase(BeanSerializerBase src, ObjectIdWriter objectIdWriter)
+    protected BeanSerializerBase(BeanSerializerBase src,
+            ObjectIdWriter objectIdWriter)
+    {
+        this(src, objectIdWriter, src._propertyFilterId);
+    }
+    
+    /**
+     * @since 2.3
+     */
+    protected BeanSerializerBase(BeanSerializerBase src,
+            ObjectIdWriter objectIdWriter, Object filterId)
     {
         super(src._handledType);
         _props = src._props;
@@ -146,7 +156,7 @@ public abstract class BeanSerializerBase
         _typeId = src._typeId;
         _anyGetterWriter = src._anyGetterWriter;
         _objectIdWriter = objectIdWriter;
-        _propertyFilterId = src._propertyFilterId;
+        _propertyFilterId = filterId;
         _serializationShape = src._serializationShape;
     }
 
@@ -185,7 +195,7 @@ public abstract class BeanSerializerBase
     }
     
     /**
-     * Fluent factory used for creating a new instance with different
+     * Mutant factory used for creating a new instance with different
      * {@link ObjectIdWriter}.
      * 
      * @since 2.0
@@ -193,7 +203,7 @@ public abstract class BeanSerializerBase
     public abstract BeanSerializerBase withObjectIdWriter(ObjectIdWriter objectIdWriter);
 
     /**
-     * Fluent factory used for creating a new instance with additional
+     * Mutant factory used for creating a new instance with additional
      * set of properties to ignore (from properties this instance otherwise has)
      * 
      * @since 2.0
@@ -201,13 +211,21 @@ public abstract class BeanSerializerBase
     protected abstract BeanSerializerBase withIgnorals(String[] toIgnore);
 
     /**
-     * Fluent factory for creating a variant that output POJO as a
+     * Mutant factory for creating a variant that output POJO as a
      * JSON Array. Implementations may ignore this request if output
      * as array is not possible (either at all, or reliably).
      * 
      * @since 2.1
      */
     protected abstract BeanSerializerBase asArraySerializer();
+
+    /**
+     * Mutant factory used for creating a new instance with different
+     * filter id (used with <code>JsonFilter</code> annotation)
+     * 
+     * @since 2.3
+     */
+    protected abstract BeanSerializerBase withFilterId(Object filterId);
     
     /**
      * Copy-constructor that is useful for sub-classes that just want to
@@ -350,7 +368,7 @@ public abstract class BeanSerializerBase
         }
         return null;
     }
-    
+
     @Override
     public JsonSerializer<?> createContextual(SerializerProvider provider,
             BeanProperty property)
@@ -358,6 +376,7 @@ public abstract class BeanSerializerBase
     {
         ObjectIdWriter oiw = _objectIdWriter;
         String[] ignorals = null;
+        Object newFilterId = null;
         final AnnotationIntrospector intr = provider.getAnnotationIntrospector();
         final AnnotatedMember accessor = (property == null || intr == null)
                 ? null : property.getMember();
@@ -369,7 +388,8 @@ public abstract class BeanSerializerBase
             if (objectIdInfo == null) {
                 // no ObjectId override, but maybe ObjectIdRef?
                 if (oiw != null) {
-                    objectIdInfo = intr.findObjectReferenceInfo(accessor, new ObjectIdInfo("", null, null));
+                    objectIdInfo = intr.findObjectReferenceInfo(accessor,
+                            new ObjectIdInfo(NAME_FOR_OBJECT_REF, null, null, null));
                     oiw = _objectIdWriter.withAlwaysAsId(objectIdInfo.getAlwaysAsId());
                 }
             } else {
@@ -386,7 +406,7 @@ public abstract class BeanSerializerBase
                 JavaType idType = provider.getTypeFactory().findTypeParameters(type, ObjectIdGenerator.class)[0];
                 // Property-based generator is trickier
                 if (implClass == ObjectIdGenerators.PropertyGenerator.class) { // most special one, needs extra work
-                    String propName = objectIdInfo.getPropertyName();
+                    String propName = objectIdInfo.getPropertyName().getSimpleName();
                     BeanPropertyWriter idProp = null;
 
                     for (int i = 0, len = _props.length ;; ++i) {
@@ -414,13 +434,22 @@ public abstract class BeanSerializerBase
                     }
                     idType = idProp.getType();
                     gen = new PropertyBasedObjectIdGenerator(objectIdInfo, idProp);
-                    oiw = ObjectIdWriter.construct(idType, null, gen, objectIdInfo.getAlwaysAsId());
+                    oiw = ObjectIdWriter.construct(idType, (PropertyName) null, gen, objectIdInfo.getAlwaysAsId());
                 } else { // other types need to be simpler
                     gen = provider.objectIdGeneratorInstance(accessor, objectIdInfo);
                     oiw = ObjectIdWriter.construct(idType, objectIdInfo.getPropertyName(), gen,
                             objectIdInfo.getAlwaysAsId());
                 }
             }
+            
+            // Or change Filter Id in use?
+            Object filterId = intr.findFilterId(accessor);
+            if (filterId != null) {
+                // but only consider case of adding a new filter id (no removal via annotation)
+                if (_propertyFilterId == null || !filterId.equals(_propertyFilterId)) {
+                    newFilterId = filterId;
+                }
+            }
         }
         // either way, need to resolve serializer:
         BeanSerializerBase contextual = this;
@@ -435,6 +464,10 @@ public abstract class BeanSerializerBase
         if (ignorals != null && ignorals.length != 0) {
             contextual = contextual.withIgnorals(ignorals);
         }
+        if (newFilterId != null) {
+            contextual = contextual.withFilterId(newFilterId);
+        }
+        
         // One more thing: are we asked to serialize POJO as array?
         JsonFormat.Shape shape = null;
         if (accessor != null) {
@@ -546,6 +579,15 @@ public abstract class BeanSerializerBase
             w.serializer.serialize(id, jgen, provider);
             return;
         }
+
+        _serializeObjectId(bean, jgen, provider, typeSer, objectId);
+    }
+
+    protected  void _serializeObjectId(Object bean,
+            JsonGenerator jgen,SerializerProvider provider,
+            TypeSerializer typeSer, WritableObjectId objectId)
+        throws IOException, JsonProcessingException, JsonGenerationException {
+        final ObjectIdWriter w = _objectIdWriter;
         String typeStr = (_typeId == null) ? null :_customTypeId(bean);
         if (typeStr == null) {
             typeSer.writeTypePrefixForObject(bean, jgen);
@@ -617,29 +659,28 @@ public abstract class BeanSerializerBase
 
     /**
      * Alternative serialization method that gets called when there is a
-     * {@link BeanPropertyFilter} that needs to be called to determine
+     * {@link PropertyFilter} that needs to be called to determine
      * which properties are to be serialized (and possibly how)
      */
-    protected void serializeFieldsFiltered(Object bean, JsonGenerator jgen, SerializerProvider provider)
+    protected void serializeFieldsFiltered(Object bean, JsonGenerator jgen,
+            SerializerProvider provider)
         throws IOException, JsonGenerationException
     {
         /* note: almost verbatim copy of "serializeFields"; copied (instead of merged)
          * so that old method need not add check for existence of filter.
          */
-        
         final BeanPropertyWriter[] props;
         if (_filteredProps != null && provider.getActiveView() != null) {
             props = _filteredProps;
         } else {
             props = _props;
         }
-        final BeanPropertyFilter filter = findFilter(provider);
+        final PropertyFilter filter = findPropertyFilter(provider, _propertyFilterId, bean);
         // better also allow missing filter actually..
         if (filter == null) {
             serializeFields(bean, jgen, provider);
             return;
         }
-        
         int i = 0;
         try {
             for (final int len = props.length; i < len; ++i) {
@@ -649,7 +690,7 @@ public abstract class BeanSerializerBase
                 }
             }
             if (_anyGetterWriter != null) {
-                _anyGetterWriter.getAndSerialize(bean, jgen, provider);
+                _anyGetterWriter.getAndFilter(bean, jgen, provider, filter);
             }
         } catch (Exception e) {
             String name = (i == props.length) ? "[anySetter]" : props[i].getName();
@@ -662,24 +703,7 @@ public abstract class BeanSerializerBase
         }
     }
 
-    /**
-     * Helper method used to locate filter that is needed, based on filter id
-     * this serializer was constructed with.
-     */
-    protected BeanPropertyFilter findFilter(SerializerProvider provider)
-        throws JsonMappingException
-    {
-        final Object filterId = _propertyFilterId;
-        FilterProvider filters = provider.getFilterProvider();
-        // Not ok to miss the provider, if a filter is declared to be needed.
-        if (filters == null) {
-            throw new JsonMappingException("Can not resolve BeanPropertyFilter with id '"+filterId+"'; no FilterProvider configured");
-        }
-        BeanPropertyFilter filter = filters.findFilter(filterId);
-        // But whether unknown ids are ok just depends on filter provider; if we get null that's fine
-        return filter;
-    }
-    
+    @Deprecated
     @Override
     public JsonNode getSchema(SerializerProvider provider, Type typeHint)
         throws JsonMappingException
@@ -698,9 +722,9 @@ public abstract class BeanSerializerBase
         //todo: should the classname go in the title?
         //o.put("title", _className);
         ObjectNode propertiesNode = o.objectNode();
-        final BeanPropertyFilter filter;
+        final PropertyFilter filter;
         if (_propertyFilterId != null) {
-            filter = findFilter(provider);
+            filter = findPropertyFilter(provider, _propertyFilterId, null);
         } else {
             filter = null;
         }
@@ -723,17 +747,22 @@ public abstract class BeanSerializerBase
         throws JsonMappingException
     {
         //deposit your output format 
-        JsonObjectFormatVisitor objectVisitor = (visitor == null) ? null : visitor.expectObjectFormat(typeHint);
-        if (objectVisitor != null) {
-            if (_propertyFilterId != null) {
-                BeanPropertyFilter filter = findFilter(visitor.getProvider());
-                for (int i = 0; i < _props.length; i++) {
-                    filter.depositSchemaProperty(_props[i], objectVisitor, visitor.getProvider());
-                }
-            } else {
-                for (int i = 0; i < _props.length; i++) {
-                    _props[i].depositSchemaProperty(objectVisitor);
-                }
+        if (visitor == null) {
+            return;
+        }
+        JsonObjectFormatVisitor objectVisitor = visitor.expectObjectFormat(typeHint);
+        if (objectVisitor == null) {
+            return;
+        }
+        if (_propertyFilterId != null) {
+            PropertyFilter filter = findPropertyFilter(visitor.getProvider(),
+                    _propertyFilterId, null);
+            for (int i = 0; i < _props.length; i++) {
+                filter.depositSchemaProperty(_props[i], objectVisitor, visitor.getProvider());
+            }
+        } else {
+            for (int i = 0; i < _props.length; i++) {
+                _props[i].depositSchemaProperty(objectVisitor);
             }
         }
     }
