@@ -5,7 +5,6 @@ import java.lang.reflect.Type;
 import java.util.*;
 
 import com.fasterxml.jackson.core.*;
-
 import com.fasterxml.jackson.databind.*;
 import com.fasterxml.jackson.databind.annotation.JacksonStdImpl;
 import com.fasterxml.jackson.databind.introspect.AnnotatedMember;
@@ -15,6 +14,7 @@ import com.fasterxml.jackson.databind.jsontype.TypeSerializer;
 import com.fasterxml.jackson.databind.node.ObjectNode;
 import com.fasterxml.jackson.databind.ser.ContainerSerializer;
 import com.fasterxml.jackson.databind.ser.ContextualSerializer;
+import com.fasterxml.jackson.databind.ser.PropertyFilter;
 import com.fasterxml.jackson.databind.ser.impl.PropertySerializerMap;
 import com.fasterxml.jackson.databind.type.TypeFactory;
 
@@ -77,6 +77,21 @@ public class MapSerializer
      * runtime value types to serializers are stored in this object.
      */
     protected PropertySerializerMap _dynamicValueSerializers;
+
+    /**
+     * Id of the property filter to use, if any; null if none.
+     *
+     * @since 2.3
+     */
+    protected final Object _filterId;
+
+    /**
+     * Flag set if output is forced to be sorted by keys (usually due
+     * to annotation).
+     * 
+     * @since 2.4
+     */
+    protected final boolean _sortKeys;
     
     /*
     /**********************************************************
@@ -100,6 +115,8 @@ public class MapSerializer
         _valueSerializer = (JsonSerializer<Object>) valueSerializer;
         _dynamicValueSerializers = PropertySerializerMap.emptyMap();
         _property = null;
+        _filterId = null;
+        _sortKeys = false;
     }
 
     @SuppressWarnings("unchecked")
@@ -117,6 +134,8 @@ public class MapSerializer
         _valueSerializer = (JsonSerializer<Object>) valueSerializer;
         _dynamicValueSerializers = src._dynamicValueSerializers;
         _property = property;
+        _filterId = src._filterId;
+        _sortKeys = src._sortKeys;
     }
 
     protected MapSerializer(MapSerializer src, TypeSerializer vts)
@@ -131,24 +150,77 @@ public class MapSerializer
         _valueSerializer = src._valueSerializer;
         _dynamicValueSerializers = src._dynamicValueSerializers;
         _property = src._property;
+        _filterId = src._filterId;
+        _sortKeys = src._sortKeys;
+    }
+
+    protected MapSerializer(MapSerializer src, Object filterId, boolean sortKeys)
+    {
+        super(Map.class, false);
+        _ignoredEntries = src._ignoredEntries;
+        _keyType = src._keyType;
+        _valueType = src._valueType;
+        _valueTypeIsStatic = src._valueTypeIsStatic;
+        _valueTypeSerializer = src._valueTypeSerializer;
+        _keySerializer = src._keySerializer;
+        _valueSerializer = src._valueSerializer;
+        _dynamicValueSerializers = src._dynamicValueSerializers;
+        _property = src._property;
+        _filterId = filterId;
+        _sortKeys = sortKeys;
     }
     
     @Override
-    public MapSerializer _withValueTypeSerializer(TypeSerializer vts)
-    {
+    public MapSerializer _withValueTypeSerializer(TypeSerializer vts) {
         return new MapSerializer(this, vts);
     }
 
+    @Deprecated // since 2.3
+    public MapSerializer withResolved(BeanProperty property,
+            JsonSerializer<?> keySerializer, JsonSerializer<?> valueSerializer, HashSet<String> ignored) {
+        return withResolved(property, keySerializer, valueSerializer, ignored, _sortKeys);
+    }
+
+    /**
+     * @since 2.4
+     */
     public MapSerializer withResolved(BeanProperty property,
             JsonSerializer<?> keySerializer, JsonSerializer<?> valueSerializer,
-            HashSet<String> ignored)
+            HashSet<String> ignored, boolean sortKeys)
     {
-        return new MapSerializer(this, property, keySerializer, valueSerializer, ignored);
+        MapSerializer ser = new MapSerializer(this, property, keySerializer, valueSerializer, ignored);
+        if (sortKeys != ser._sortKeys) {
+            ser = new MapSerializer(ser, _filterId, sortKeys);
+        }
+        return ser;
     }
     
+    /**
+     * @since 2.3
+     */
+    public MapSerializer withFilterId(Object filterId) {
+        return (_filterId == filterId) ? this : new MapSerializer(this, filterId, _sortKeys);
+    }
+
+    /**
+     * @deprecated Since 2.3 use the method that takes `filterId`
+     */
+    @Deprecated
     public static MapSerializer construct(String[] ignoredList, JavaType mapType,
             boolean staticValueType, TypeSerializer vts,
             JsonSerializer<Object> keySerializer, JsonSerializer<Object> valueSerializer)
+    {
+        return construct(ignoredList, mapType, staticValueType, vts,
+                keySerializer, valueSerializer, null);
+    }
+
+    /**
+     * @since 2.3
+     */
+    public static MapSerializer construct(String[] ignoredList, JavaType mapType,
+            boolean staticValueType, TypeSerializer vts,
+            JsonSerializer<Object> keySerializer, JsonSerializer<Object> valueSerializer,
+            Object filterId)
     {
         HashSet<String> ignoredEntries = toSet(ignoredList);
         JavaType keyType, valueType;
@@ -162,9 +234,18 @@ public class MapSerializer
         // If value type is final, it's same as forcing static value typing:
         if (!staticValueType) {
             staticValueType = (valueType != null && valueType.isFinal());
+        } else {
+            // also: Object.class can not be handled as static, ever
+            if (valueType.getRawClass() == Object.class) {
+                staticValueType = false;
+            }
         }
-        return new MapSerializer(ignoredEntries, keyType, valueType, staticValueType, vts,
+        MapSerializer ser = new MapSerializer(ignoredEntries, keyType, valueType, staticValueType, vts,
                 keySerializer, valueSerializer);
+        if (filterId != null) {
+            ser = ser.withFilterId(filterId);
+        }
+        return ser;
     }
 
     private static HashSet<String> toSet(String[] ignoredEntries) {
@@ -177,7 +258,7 @@ public class MapSerializer
         }
         return result;
     }
-    
+
     /*
     /**********************************************************
     /* Post-processing (contextualization)
@@ -195,57 +276,67 @@ public class MapSerializer
          */
         JsonSerializer<?> ser = null;
         JsonSerializer<?> keySer = null;
+        final AnnotationIntrospector intr = provider.getAnnotationIntrospector();
+        final AnnotatedMember propertyAcc = (property == null) ? null : property.getMember();
 
         // First: if we have a property, may have property-annotation overrides
-        if (property != null) {
-            AnnotatedMember m = property.getMember();
-            if (m != null) {
-                Object serDef;
-                final AnnotationIntrospector intr = provider.getAnnotationIntrospector();
-                serDef = intr.findKeySerializer(m);
-                if (serDef != null) {
-                    keySer = provider.serializerInstance(m, serDef);
-                }
-                serDef = intr.findContentSerializer(m);
-                if (serDef != null) {
-                    ser = provider.serializerInstance(m, serDef);
-                }
+        if (propertyAcc != null && intr != null) {
+            Object serDef = intr.findKeySerializer(propertyAcc);
+            if (serDef != null) {
+                keySer = provider.serializerInstance(propertyAcc, serDef);
+            }
+            serDef = intr.findContentSerializer(propertyAcc);
+            if (serDef != null) {
+                ser = provider.serializerInstance(propertyAcc, serDef);
             }
         }
         if (ser == null) {
             ser = _valueSerializer;
         }
-        // #124: May have a content converter
+        // [Issue#124]: May have a content converter
         ser = findConvertingContentSerializer(provider, property, ser);
         if (ser == null) {
             // 30-Sep-2012, tatu: One more thing -- if explicit content type is annotated,
             //   we can consider it a static case as well.
-            if (_valueTypeIsStatic || hasContentTypeAnnotation(provider, property)) {
+            // 20-Aug-2013, tatu: Need to avoid trying to access serializer for java.lang.Object tho
+            if ((_valueTypeIsStatic && _valueType.getRawClass() != Object.class)
+                    || hasContentTypeAnnotation(provider, property)) {
                 ser = provider.findValueSerializer(_valueType, property);
             }
-        } else if (ser instanceof ContextualSerializer) {
-            ser = ((ContextualSerializer) ser).createContextual(provider, property);
+        } else {
+            ser = provider.handleSecondaryContextualization(ser, property);
         }
         if (keySer == null) {
             keySer = _keySerializer;
         }
         if (keySer == null) {
             keySer = provider.findKeySerializer(_keyType, property);
-        } else if (keySer instanceof ContextualSerializer) {
-            keySer = ((ContextualSerializer) keySer).createContextual(provider, property);
+        } else {
+            keySer = provider.handleSecondaryContextualization(keySer, property);
         }
-        HashSet<String> ignored = this._ignoredEntries;
-        AnnotationIntrospector intr = provider.getAnnotationIntrospector();
-        if (intr != null && property != null) {
-            String[] moreToIgnore = intr.findPropertiesToIgnore(property.getMember());
+        HashSet<String> ignored = _ignoredEntries;
+        boolean sortKeys = false;
+        if (intr != null && propertyAcc != null) {
+            String[] moreToIgnore = intr.findPropertiesToIgnore(propertyAcc);
             if (moreToIgnore != null) {
                 ignored = (ignored == null) ? new HashSet<String>() : new HashSet<String>(ignored);
                 for (String str : moreToIgnore) {
                     ignored.add(str);
                 }
             }
+            Boolean b = intr.findSerializationSortAlphabetically(propertyAcc);
+            sortKeys = (b != null) && b.booleanValue();
         }
-        return withResolved(property, keySer, ser, ignored);
+        MapSerializer mser = withResolved(property, keySer, ser, ignored, sortKeys);
+
+        // [Issue#307]: allow filtering
+        if (property != null) {
+            Object filterId = intr.findFilterId(property.getMember());
+            if (filterId != null) {
+                mser = mser.withFilterId(filterId);
+            }
+        }
+        return mser;
     }
     
     /*
@@ -306,7 +397,13 @@ public class MapSerializer
     {
         jgen.writeStartObject();
         if (!value.isEmpty()) {
-            if (provider.isEnabled(SerializationFeature.ORDER_MAP_ENTRIES_BY_KEYS)) {
+            if (_filterId != null) {
+                serializeFilteredFields(value, jgen, provider,
+                        findPropertyFilter(provider, _filterId, value));
+                jgen.writeEndObject();
+                return;
+            }
+            if (_sortKeys || provider.isEnabled(SerializationFeature.ORDER_MAP_ENTRIES_BY_KEYS)) {
                 value = _orderEntries(value);
             }
             if (_valueSerializer != null) {
@@ -314,7 +411,7 @@ public class MapSerializer
             } else {
                 serializeFields(value, jgen, provider);
             }
-        }        
+        }
         jgen.writeEndObject();
     }
 
@@ -325,7 +422,7 @@ public class MapSerializer
     {
         typeSer.writeTypePrefixForObject(value, jgen);
         if (!value.isEmpty()) {
-            if (provider.isEnabled(SerializationFeature.ORDER_MAP_ENTRIES_BY_KEYS)) {
+            if (_sortKeys || provider.isEnabled(SerializationFeature.ORDER_MAP_ENTRIES_BY_KEYS)) {
                 value = _orderEntries(value);
             }
             if (_valueSerializer != null) {
@@ -408,7 +505,7 @@ public class MapSerializer
      */
     protected void serializeFieldsUsing(Map<?,?> value, JsonGenerator jgen, SerializerProvider provider,
             JsonSerializer<Object> ser)
-            throws IOException, JsonGenerationException
+        throws IOException, JsonGenerationException
     {
         final JsonSerializer<Object> keySerializer = _keySerializer;
         final HashSet<String> ignored = _ignoredEntries;
@@ -444,6 +541,66 @@ public class MapSerializer
         }
     }
 
+    /**
+     * Helper method used when we have a JSON Filter to use for potentially
+     * filtering out Map entries.
+     *<p>
+     * NOTE: initially only called externally, by <code>AnyGetterWriter</code>
+     * 
+     * @since 2.3
+     */
+    public void serializeFilteredFields(Map<?,?> value, JsonGenerator jgen, SerializerProvider provider,
+            PropertyFilter filter)
+        throws IOException, JsonGenerationException
+    {
+        final HashSet<String> ignored = _ignoredEntries;
+        final boolean skipNulls = !provider.isEnabled(SerializationFeature.WRITE_NULL_MAP_VALUES);
+
+        PropertySerializerMap serializers = _dynamicValueSerializers;
+        final MapProperty prop = new MapProperty(_valueTypeSerializer);
+
+        for (Map.Entry<?,?> entry : value.entrySet()) {
+            // First, serialize key
+            final Object keyElem = entry.getKey();
+            final Object valueElem = entry.getValue();
+            JsonSerializer<Object> keySer;
+            if (keyElem == null) {
+                keySer = provider.findNullKeySerializer(_keyType, _property);
+            } else {
+                // [JACKSON-314] skip entries with null values?
+                if (skipNulls && valueElem == null) continue;
+                // One twist: is entry ignorable? If so, skip
+                if (ignored != null && ignored.contains(keyElem)) continue;
+                keySer = _keySerializer;
+            }
+            JsonSerializer<Object> valueSer;
+            // And then value
+            if (valueElem == null) {
+                valueSer = provider.getDefaultNullValueSerializer();
+            } else {
+                Class<?> cc = valueElem.getClass();
+                valueSer = serializers.serializerFor(cc);
+                if (valueSer == null) {
+                    if (_valueType.hasGenericTypes()) {
+                        valueSer = _findAndAddDynamic(serializers,
+                                provider.constructSpecializedType(_valueType, cc), provider);
+                    } else {
+                        valueSer = _findAndAddDynamic(serializers, cc, provider);
+                    }
+                    serializers = _dynamicValueSerializers;
+                }
+            }
+            prop.reset(keyElem, valueElem, keySer, valueSer);
+            try {
+                filter.serializeAsField(value, jgen, provider, prop);
+            } catch (Exception e) {
+                // [JACKSON-55] Need to add reference information
+                String keyDesc = ""+keyElem;
+                wrapAndThrow(provider, e, value, keyDesc);
+            }
+        }
+    }
+    
     protected void serializeTypedFields(Map<?,?> value, JsonGenerator jgen, SerializerProvider provider)
         throws IOException, JsonGenerationException
     {
@@ -476,7 +633,11 @@ public class MapSerializer
                 if (cc == prevValueClass) {
                     currSerializer = prevValueSerializer;
                 } else {
-                    currSerializer = provider.findValueSerializer(cc, _property);
+                    if (_valueType.hasGenericTypes()) {
+                        currSerializer = provider.findValueSerializer(provider.constructSpecializedType(_valueType, cc), _property);
+                    } else {
+                        currSerializer = provider.findValueSerializer(cc, _property);
+                    }
                     prevValueSerializer = currSerializer;
                     prevValueClass = cc;
                 }
@@ -525,7 +686,7 @@ public class MapSerializer
     protected final JsonSerializer<Object> _findAndAddDynamic(PropertySerializerMap map,
             Class<?> type, SerializerProvider provider) throws JsonMappingException
     {
-        PropertySerializerMap.SerializerAndMapResult result = map.findAndAddSerializer(type, provider, _property);
+        PropertySerializerMap.SerializerAndMapResult result = map.findAndAddSecondarySerializer(type, provider, _property);
         // did we get a new map of serializers? If so, start using it
         if (map != result.map) {
             _dynamicValueSerializers = result.map;
@@ -536,7 +697,7 @@ public class MapSerializer
     protected final JsonSerializer<Object> _findAndAddDynamic(PropertySerializerMap map,
             JavaType type, SerializerProvider provider) throws JsonMappingException
     {
-        PropertySerializerMap.SerializerAndMapResult result = map.findAndAddSerializer(type, provider, _property);
+        PropertySerializerMap.SerializerAndMapResult result = map.findAndAddSecondarySerializer(type, provider, _property);
         if (map != result.map) {
             _dynamicValueSerializers = result.map;
         }
