@@ -8,11 +8,10 @@ public class TestStdDateFormat
     extends BaseMapTest
 {
     public void testFactories() {
-        assertNotNull(StdDateFormat.getBlueprintISO8601Format());
-        assertNotNull(StdDateFormat.getBlueprintRFC1123Format());
         TimeZone tz = TimeZone.getTimeZone("GMT");
-        assertNotNull(StdDateFormat.getISO8601Format(tz));
-        assertNotNull(StdDateFormat.getRFC1123Format(tz));
+        Locale loc = Locale.US;
+        assertNotNull(StdDateFormat.getISO8601Format(tz, loc));
+        assertNotNull(StdDateFormat.getRFC1123Format(tz, loc));
     }
 
     public void testInvalid() {
