@@ -109,8 +109,9 @@ public class TestValueInstantiator extends BaseMapTest
         @Override
         public CreatorProperty[] getFromObjectArguments(DeserializationConfig config) {
             return  new CreatorProperty[] {
-                    new CreatorProperty("type", config.constructType(Class.class), null,
-                            null, null, null, 0, null, true)
+                    new CreatorProperty(new PropertyName("type"), config.constructType(Class.class), null,
+                            null, null, null, 0, null,
+                            PropertyMetadata.STD_REQUIRED)
             };
         }
 
@@ -138,8 +139,9 @@ public class TestValueInstantiator extends BaseMapTest
         @Override
         public CreatorProperty[] getFromObjectArguments(DeserializationConfig config) {
             return  new CreatorProperty[] {
-                    new CreatorProperty("name", config.constructType(String.class), null,
-                            null, null, null, 0, null, true)
+                    new CreatorProperty(new PropertyName("name"), config.constructType(String.class), null,
+                            null, null, null, 0, null,
+                            PropertyMetadata.STD_REQUIRED)
             };
         }
 
@@ -366,8 +368,9 @@ public class TestValueInstantiator extends BaseMapTest
                     @Override
                     public CreatorProperty[] getFromObjectArguments(DeserializationConfig config) {
                         return  new CreatorProperty[] {
-                                new CreatorProperty("secret", config.constructType(String.class), null,
-                                        null, null, null, 0, null, true)
+                                new CreatorProperty(new PropertyName("secret"), config.constructType(String.class), null,
+                                        null, null, null, 0, null,
+                                        PropertyMetadata.STD_REQUIRED)
                         };
                     }
         
